#$Id$#

from books.model.PreferenceList import PreferenceList
from books.model.Preference import Preference
from books.model.CustomField import CustomField
from books.model.PlaceholderAddressFormat import PlaceholderAddressFormat
from books.model.Autoreminder import Autoreminder
from books.model.AutoReminderList import AutoReminderList
from books.model.Term import Term
from books.model.AddressFormat import AddressFormat
from books.model.Invoice import Invoice
from books.model.Estimate import Estimate
from books.model.Contact import Contact
from books.model.Organization import Organization
from books.model.Customer import Customer
from books.model.Organization import Organization
from books.model.Address import Address
from books.model.UserList import UserList
from books.model.User import User
from books.model.EmailId import EmailId
from books.model.PageContext import PageContext
from books.model.Item import Item
from books.model.ItemList import ItemList
from books.model.InvoiceSetting import InvoiceSetting
from books.model.NotesAndTerms import NotesAndTerms
from books.model.EstimateSetting import EstimateSetting
from books.model.CreditnoteSetting import CreditnoteSetting
from books.model.CurrencyList import CurrencyList
from books.model.Currency import Currency
from books.model.ExchangeRate import ExchangeRate
from books.model.ExchangeRateList import ExchangeRateList
from books.model.TaxList import TaxList
from books.model.Tax import Tax
from books.model.TaxGroup import TaxGroup
from books.model.OpeningBalance import OpeningBalance
from books.model.Account import Account
from books.model.PlaceHolder import PlaceHolder
from books.model.ManualReminder import ManualReminder
from books.model.ManualReminderList import ManualReminderList

class SettingsParser:
    """This class is used to parse the json for Settings."""

    def preference_list(self, resp):
        """This method parses the given response and returns preferneces 
            object.

        Args:
            resp(dict): Response containing json obejct for preferences.

        Returns:
            instance: Prefenreces object.

        Raises:
            Books Exception: If status is not '200' or '201'.

        """
        preferences = resp['preferences']
        preferences_list = PreferenceList()
        preference_obj = Preference()
        preference_obj.set_convert_to_invoice(preferences[\
        'convert_to_invoice'])
        preference_obj.set_attach_pdf_for_email(preferences[\
        'attach_pdf_for_email'])
        preference_obj.set_estimate_approval_status(preferences[\
        'estimate_approval_status'])
        preference_obj.set_notify_me_on_online_payment(preferences[\
        'notify_me_on_online_payment'])
        preference_obj.set_send_payment_receipt_acknowledgement(preferences[\
        'send_payment_receipt_acknowledgement'])
        preference_obj.set_auto_notify_recurring_invoice(preferences[\
        'auto_notify_recurring_invoice'])
        preference_obj.set_snail_mail_include_payment_stub(preferences[\
        'snail_mail_include_payment_stub'])
        preference_obj.set_is_show_powered_by(preferences['is_show_powered_by'])
        preference_obj.set_attach_expense_receipt_to_invoice(preferences[\
        'attach_expense_receipt_to_invoice'])
        preference_obj.set_is_estimate_enabled(preferences[\
        'is_estimate_enabled'])
        preference_obj.set_is_project_enabled(preferences['is_project_enabled'])
        preference_obj.set_is_purchaseorder_enabled(preferences[\
        'is_purchaseorder_enabled'])
        preference_obj.set_is_salesorder_enabled(preferences[\
        'is_salesorder_enabled'])
        preference_obj.set_is_pricebooks_enabled(preferences[\
        'is_pricebooks_enabled'])
        preference_obj.set_attach_payment_receipt_with_acknowledgement(\
        preferences['attach_payment_receipt_with_acknowledgement'])
        for value in preferences['auto_reminders']:
            auto_reminder = Autoreminder()
            auto_reminder.set_payment_reminder_id(value['payment_reminder_id'])
            auto_reminder.set_is_enabled(value['is_enabled'])
            auto_reminder.set_notification_type(value['notification_type'])
            auto_reminder.set_address_type(value['address_type'])
            auto_reminder.set_number_of_days(value['number_of_days'])
            auto_reminder.set_subject(value['subject'])
            auto_reminder.set_body(value['body'])
            preference_obj.set_auto_reminders(auto_reminder)
        terms = preferences['terms']
        terms_obj = Term()
        terms_obj.set_invoice_terms(terms['invoice_terms'])
        terms_obj.set_estimate_terms(terms['estimate_terms'])
        terms_obj.set_creditnote_terms(terms['creditnote_terms'])
        preference_obj.set_terms(terms_obj)
        address_formats_obj = AddressFormat()
        address_formats = preferences['address_formats']
        address_formats_obj.set_organization_address_format(address_formats[\
        'organization_address_format'])
        address_formats_obj.set_customer_address_format(address_formats[\
        'customer_address_format'])
        preference_obj.set_address_formats(address_formats_obj)
        preferences_list.set_preferences(preference_obj)
        custom_fields = resp['customfields']
        custom_fields_obj = CustomField()
        for value in custom_fields['invoice']:
            invoice = Invoice()
            invoice.set_index(value['index'])
            invoice.set_show_in_all_pdf(value['show_in_all_pdf'])
            invoice.set_label(value['label'])
            custom_fields_obj.set_invoice(invoice)
        for value in custom_fields['contact']:
            contact = Contact()
            contact.set_index(value['index'])
            contact.set_show_in_all_pdf(vlaue['show_in_all_pdf'])
            contact.set_label(value['label'])
            custom_fields_obj.set_contact(contact)
        for value in custom_fields['estimate']:
            estimate = Estimate()
            estimate.set_index(value['index'])
            estimate.set_show_in_all_pdf(value['show_in_all_pdf'])
            estimate.set_label(value['label'])
            custom_fields_obj.set_estimate(estimate)
        preferences_list.set_custom_fields(custom_fields_obj)
        placeholders_address_format = resp['placeholders_address_format']
        placeholders_address_format_obj = PlaceholderAddressFormat()
        for value in placeholders_address_format['organization']:
            organization = Organization()
            organization.set_value(value['value'])
            organization.set_name(value['name'])
            placeholders_address_format_obj.set_organization(organization)
        for value in placeholders_address_format['customer']:
            customer = Customer()
            customer.set_name(value['name'])
            customer.set_value(value['value'])
            placeholders_address_format_obj.set_customer(customer)
        preferences_list.set_placeholders_address_format(\
        placeholders_address_format_obj)
        return preferences_list

    def get_message(self, resp):
        """This message parses the given response and returns message string.
    
        Args:
            resp(dict): Response containing json object for message.

        Returns:
            str: Success message.

        """
        return resp['message']

    def get_organizations(self, resp):
        """This message parses the given response and returns organizations 
            list.

        Args:
            resp(dict): Response containing json object for organizations list.
 
        Returns:
            instance: Organizations list object.

        """
        organizations_list = []
        for value in resp['organizations']:
            organization = Organization()
            organization.set_organization_id(value['organization_id'])
            organization.set_name(value['name'])
            organization.set_contact_name(value['contact_name'])
            organization.set_email(value['email'])
            organization.set_is_default_org(value['is_default_org'])
            organization.set_version(value['version'])
            organization.set_plan_type(value['plan_type'])
            organization.set_tax_group_enabled(value['tax_group_enabled'])
            organization.set_plan_name(value['plan_name'])
            organization.set_plan_period(value['plan_period'])
            organization.set_language_code(value['language_code'])
            organization.set_fiscal_year_start_month(value[\
            'fiscal_year_start_month'])
            organization.set_account_created_date(value[\
            'account_created_date'])
            organization.set_account_created_date_formatted(value[\
            'account_created_date_formatted'])
            organization.set_time_zone(value['time_zone'])
            organization.set_is_org_active(value['is_org_active'])
            organization.set_currency_id(value['currency_id'])
            organization.set_currency_code(value['currency_code'])
            organization.set_currency_symbol(value['currency_symbol'])
            organization.set_currency_format(value['currency_format'])
            organization.set_price_precision(value['price_precision'])
            organizations_list.append(organization)
        return organizations_list

    def get_organization(self, resp):
        """This method parses the given response and returns organization 
            object.

        Args:
            resp(dict): Response containing json object for organization.

        """
        organization = resp['organization']
        organization_obj = Organization()
        organization_obj.set_organization_id(organization['organization_id'])
        organization_obj.set_name(organization['name'])
        organization_obj.set_is_default_org(organization['is_default_org'])
        organization_obj.set_user_role(organization['user_role'])
        organization_obj.set_account_created_date(organization[\
        'account_created_date'])
        organization_obj.set_time_zone(organization['time_zone'])
        organization_obj.set_language_code(organization['language_code'])
        organization_obj.set_date_format(organization['date_format'])
        organization_obj.set_field_separator(organization['field_separator'])
        organization_obj.set_fiscal_year_start_month(organization[\
        'fiscal_year_start_month'])
        organization_obj.set_contact_name(organization['contact_name'])
        organization_obj.set_industry_type(organization['industry_type'])
        organization_obj.set_industry_size(organization['industry_size'])
        organization_obj.set_company_id_label(organization['company_id_label'])
        organization_obj.set_company_id_value(organization['company_id_value'])
        organization_obj.set_tax_id_label(organization['tax_id_label'])
        organization_obj.set_tax_id_value(organization['tax_id_value'])
        organization_obj.set_currency_id(organization['currency_id'])
        organization_obj.set_currency_code(organization['currency_code'])
        organization_obj.set_currency_symbol(organization['currency_symbol'])
        organization_obj.set_currency_format(organization['currency_format'])
        organization_obj.set_price_precision(organization['price_precision'])
        address = organization['address']
        address_obj = Address()
        address_obj.set_street_address1(address['street_address1'])
        address_obj.set_street_address2(address['street_address2'])
        address_obj.set_city(address['city'])
        address_obj.set_state(address['state'])
        address_obj.set_country(address['country'])
        address_obj.set_zip(address['zip'])
        organization_obj.set_address(address_obj)
        organization_obj.set_org_address(organization['org_address'])
        organization_obj.set_remit_to_address(organization['remit_to_address'])
        organization_obj.set_phone(organization['phone'])
        organization_obj.set_fax(organization['fax'])
        organization_obj.set_website(organization['website'])
        organization_obj.set_email(organization['email'])
        organization_obj.set_tax_basis(organization['tax_basis'])
        for value in organization['custom_fields']:
            custom_field = CustomField()
            custom_field.set_value(value['value'])
            custom_field.set_index(value['index'])
            custom_field.set_label(value['label'])
            organization_obj.set_custom_fields(custom_field)
        organization_obj.set_is_org_active(organization['is_org_active'])
        organization_obj.set_is_new_customer_custom_fields(organization[\
        'is_new_customer_custom_fields'])
        organization_obj.set_is_portal_enabled(organization['is_portal_enabled'])
        organization_obj.set_portal_name(organization['portal_name'])
        return organization_obj

    def get_users(self, resp):
        """This method parses the given response and returns USers list object.

        Args:
            resp(dict): Response containing json object for users list.

        Returns:
            instance: Users list object.

        """
        users_list = UserList()
        for value in resp['users']:
            user = User()
            user.set_user_id(value['user_id'])
            user.set_role_id(value['role_id'])
            user.set_name(value['name'])
            user.set_email(value['email'])
            user.set_user_role(value['user_role'])
            user.set_status(value['status'])
            user.set_is_current_user(value['is_current_user'])
            user.set_photo_url(value['photo_url'])
            users_list.set_users(user)
        page_context_obj = PageContext()
        page_context = resp['page_context']
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        page_context_obj.set_applied_filter(page_context['applied_filter'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        users_list.set_page_context(page_context_obj)
        return users_list

    def get_user(self, resp):
        """This method parses the given response and returns user object.

        Args:
            resp(dict): Response containing json object for user.

        Returns:
            instance: User object.

        Raises: 
            Books Exception: If status is not '200' or '201'.

        """
        user = resp['user'] 
        user_obj = User()
        user_obj.set_user_id(user['user_id'])
        user_obj.set_name(user['name'])
        for value in user['email_ids']:
            email_id = EmailId()
            email_id.set_is_selected(value['is_selected'])
            email_id.set_email(value['email'])
            user_obj.set_email_ids(email_id)
        user_obj.set_status(user['status'])
        user_obj.set_user_role(user['user_role'])
        user_obj.set_created_time(user['created_time'])
        user_obj.set_photo_url(user['photo_url'])
        return user_obj

    def get_items(self, resp):
        """This method parses the given response and returns items list object.

        Args:
            resp(dict): Response containing json object for items list.

        Returns:
            instance: Items list object.

        """
        items_list = ItemList()
        for value in resp['items']:
            item = Item()
            item.set_item_id(value['item_id'])
            item.set_name(value['name'])
            item.set_status(value['status'])
            item.set_description(value['description'])
            item.set_rate(value['rate'])
            item.set_tax_id(value['tax_id'])
            item.set_tax_name(value['tax_name'])
            item.set_tax_percentage(value['tax_percentage'])
            for tp in value['item_tax_preferences']:
                item.set_item_tax_preferences(tp)
            items_list.set_items(item)
        page_context_obj = PageContext()
        page_context = resp['page_context']
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        page_context_obj.set_applied_filter(page_context['applied_filter'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        items_list.set_page_context(page_context_obj)
        return items_list

    def get_item(self, resp):
        """This method parses the given response and returns item object.

        Args:
            resp(dict): Response containing json object for item.

        Returns:
            instance: Item object.

        """
        item = resp['item']
        item_obj = Item()
        item_obj.set_item_id(item['item_id'])
        item_obj.set_name(item['name'])
        item_obj.set_status(item['status'])
        item_obj.set_description(item['description'])
        item_obj.set_rate(item['rate'])
        item_obj.set_unit(item['unit'])
        item_obj.set_account_id(item['account_id'])
        item_obj.set_account_name(item['account_name'])
        item_obj.set_tax_id(item['tax_id'])
        item_obj.set_tax_name(item['tax_name'])
        item_obj.set_tax_percentage(item['tax_percentage'])
        item_obj.set_tax_type(item['tax_type'])
        return item_obj
        
    def get_invoice_settings(self, resp):
        """This method parses the given response and returns invoice settings 
            object.

        Args:
            resp(dict): Response containing json object for invoice settings.

        Returns:
            instance: Invoice settings object.

        """
        invoice_settings = resp['invoice_settings']
        invoice_settings_obj = InvoiceSetting()
        invoice_settings_obj.set_auto_generate(invoice_settings[\
        'auto_generate'])
        invoice_settings_obj.set_prefix_string(invoice_settings[\
        'prefix_string'])
        invoice_settings_obj.set_start_at(invoice_settings['start_at'])
        invoice_settings_obj.set_next_number(invoice_settings['next_number'])
        invoice_settings_obj.set_quantity_precision(invoice_settings[\
        'quantity_precision'])
        invoice_settings_obj.set_discount_type(invoice_settings[\
        'discount_type'])
        invoice_settings_obj.set_is_discount_before_tax(invoice_settings[\
        'is_discount_before_tax'])
        invoice_settings_obj.set_reference_text(invoice_settings[\
        'reference_text'])
        invoice_settings_obj.set_notes(invoice_settings['notes'])
        invoice_settings_obj.set_terms(invoice_settings['terms'])
        invoice_settings_obj.set_is_shipping_charge_required(invoice_settings[\
        'is_shipping_charge_required'])
        invoice_settings_obj.set_is_adjustment_required(invoice_settings[\
        'is_adjustment_required'])
        invoice_settings_obj.set_is_open_invoice_editable(invoice_settings[\
        'is_open_invoice_editable'])
        invoice_settings_obj.set_warn_convert_to_open(invoice_settings[\
        'warn_convert_to_open']) 
        invoice_settings_obj.set_warn_create_creditnotes(invoice_settings[\
        'warn_create_creditnotes'])
        invoice_settings_obj.set_attach_expense_receipt_to_invoice(\
        invoice_settings['attach_expense_receipt_to_invoice'])
        invoice_settings_obj.set_invoice_item_type(invoice_settings[\
        'invoice_item_type'])
        invoice_settings_obj.set_is_sales_person_required(invoice_settings[\
        'is_sales_person_required'])
        return invoice_settings_obj

    def get_notes_and_terms(self, resp):
        """This method parses the given response and returns notes and terms 
            object.
    
        Args:
            resp(dict): Dictionary containing json object for estimate 
                settings.

        Returns:
            instance: Notes and terms object.

        """
        notes_and_terms = resp['notes_and_terms']
        notes_and_terms_obj = NotesAndTerms()
        notes_and_terms_obj.set_notes(notes_and_terms['notes'])
        notes_and_terms_obj.set_terms(notes_and_terms['terms'])
        return notes_and_terms_obj

    def get_estimate_settings(self, resp):
        """This method parses the given response and returns estimate settings 
            object.

        Args:
            resp: Dictionary containing json object for estimate settings.

        Returns:
            instance: Estimate settings object.

        """
        estimate_settings = resp['estimate_settings']
        estimate_settings_obj = EstimateSetting()
        estimate_settings_obj.set_auto_generate(estimate_settings[\
        'auto_generate'])
        estimate_settings_obj.set_prefix_string(estimate_settings[\
        'prefix_string'])
        estimate_settings_obj.set_start_at(estimate_settings['start_at']) 
        estimate_settings_obj.set_next_number(estimate_settings['next_number'])
        estimate_settings_obj.set_quantity_precision(estimate_settings[\
        'quantity_precision'])
        estimate_settings_obj.set_discount_type(estimate_settings[\
        'discount_type'])
        estimate_settings_obj.set_is_discount_before_tax(estimate_settings[\
        'is_discount_before_tax']) 
        estimate_settings_obj.set_reference_text(estimate_settings[\
        'reference_text'])
        estimate_settings_obj.set_notes(estimate_settings['notes'])
        estimate_settings_obj.set_terms(estimate_settings['terms'])
        estimate_settings_obj.set_terms_to_invoice(estimate_settings[\
        'terms_to_invoice'])
        estimate_settings_obj.set_notes_to_invoice(estimate_settings[\
        'notes_to_invoice'])
        estimate_settings_obj.set_warn_estimate_to_invoice(estimate_settings[\
        'warn_estimate_to_invoice'])
        estimate_settings_obj.set_is_sales_person_required(estimate_settings[\
        'is_sales_person_required'])
        return estimate_settings_obj

    def get_creditnote_settings(self, resp):
        """This method parses the given response and returns creditnote 
            settings.

        Args:
            resp(dict): Dictionary containing json object for creditnote 
                settings.

        Returns:
            instance: Creditnotes settings object.

        """
        creditnote_settings_obj = CreditnoteSetting()
        creditnote_settings = resp['creditnote_settings'] 
        creditnote_settings_obj.set_auto_generate(creditnote_settings[\
        'auto_generate'])
        creditnote_settings_obj.set_prefix_string(creditnote_settings[\
        'prefix_string'])
        creditnote_settings_obj.set_reference_text(creditnote_settings[\
        'reference_text'])
        creditnote_settings_obj.set_next_number(creditnote_settings[\
        'next_number'])
        creditnote_settings_obj.set_notes(creditnote_settings['notes'])
        creditnote_settings_obj.set_terms(creditnote_settings['terms'])
        return creditnote_settings_obj

    def get_currencies(self, resp):
        """This method parses the given response and returns currency list 
            object.

        Args:
            resp(dict): Response containing json object for list of currencies.

        Returns:
            instance: Currency list object.

        """
        currency_list = CurrencyList()
        for value in resp['currencies']:
            currency = Currency()
            currency.set_currency_id(value['currency_id'])
            currency.set_currency_code(value['currency_code'])
            currency.set_currency_name(value['currency_name'])
            currency.set_currency_symbol(value['currency_symbol'])
            currency.set_price_precision(value['price_precision'])
            currency.set_currency_format(value['currency_format'])
            currency.set_is_base_currency(value['is_base_currency'])
            currency.set_exchange_rate(value['exchange_rate'])
            currency.set_effective_date(value['effective_date'])
            currency_list.set_currencies(currency)
        page_context_obj = PageContext()
        page_context = resp['page_context'] 
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        currency_list.set_page_context(page_context_obj)
        return currency_list

    def get_currency(self, resp):
        """This method parses the given response and returns currency object.

        Args:
            resp(dict): Response containing json object for currency.

        Returns:
            instance: Currency object.

        """
        currency_obj = Currency()
        currency = resp['currency'] 
        currency_obj.set_currency_id(currency['currency_id'])
        currency_obj.set_currency_code(currency['currency_code'])
        currency_obj.set_currency_name(currency['currency_name'])
        currency_obj.set_currency_symbol(currency['currency_symbol'])
        currency_obj.set_price_precision(currency['price_precision'])
        currency_obj.set_currency_format(currency['currency_format'])
        currency_obj.set_is_base_currency(currency['is_base_currency'])
        return currency_obj

    def get_exchange_rates(self, resp):
        """This method parses the given response and returns exchange rates 
            list.

        Args:
            resp(dict): Response containing json object for exchange rate.
 
        Returns:
            list of instance: List of exchange rates object.

        """
        exchange_rates = ExchangeRateList()
        for value in resp['exchange_rates']:
            exchange_rate = ExchangeRate()
            exchange_rate.set_exchange_rate_id(value['exchange_rate_id'])
            exchange_rate.set_currency_id(value['currency_id'])
            exchange_rate.set_currency_code(value['currency_code'])
            exchange_rate.set_effective_date(value['effective_date']) 
            exchange_rate.set_rate(value['rate'])
            exchange_rates.set_exchange_rates(exchange_rate)
        return exchange_rates

    def get_exchange_rate(self, resp):
        """This method parses the given response and returns exchange rate 
            object.

        Args:
            resp(dict): Response containing json object for exchange rate 
                object.

        Returns: 
            instance: Exchange rate object.

        """
        exchange_rate = resp['exchange_rate']
        exchange_rate_obj = ExchangeRate()
        exchange_rate_obj.set_exchange_rate_id(exchange_rate[\
        'exchange_rate_id'])
        exchange_rate_obj.set_currency_id(exchange_rate['currency_id'])
        exchange_rate_obj.set_currency_code(exchange_rate['currency_code'])
        exchange_rate_obj.set_effective_date(exchange_rate['effective_date'])
        exchange_rate_obj.set_rate(exchange_rate['rate'])
        return exchange_rate_obj   

    def get_taxes(self, resp):
        """This method parses the given response and returns tax list object.

        Args:
            resp(dict): Response containing json object for taxes.

        Returns:
            instance: Tax list object.

        """
        tax_list = TaxList()
        for value in resp['taxes']:
            tax = Tax()
            tax.set_tax_id(value['tax_id'])
            tax.set_tax_name(value['tax_name'])
            tax.set_tax_percentage(value['tax_percentage'])
            tax.set_tax_type(value['tax_type'])
            tax_list.set_taxes(tax)
        page_context = resp['page_context']
        page_context_obj = PageContext()
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        tax_list.set_page_context(page_context_obj)
        return tax_list

    def get_tax(self, resp):
        """This method parses the given response and returns tax object.

        Args:
            resp(dict): Response containing json object for tax.

        Returns:
            instance: Tax object.

        """
        tax_obj = Tax()
        tax = resp['tax']
        tax_obj.set_tax_id(tax['tax_id'])
        tax_obj.set_tax_name(tax['tax_name'])
        tax_obj.set_tax_percentage(tax['tax_percentage'])
        tax_obj.set_tax_type(tax['tax_type'])
        return tax_obj

    def get_tax_group(self, resp):
        """This method parses the given response and returns Tax group object.

        Args:
            resp(dict): Response containing json object for tax group.

        Returns:
            instance: Tax group object.

        Raises:
            Books Exception: If status is not '200' or '201'.

        """
        tax_group_obj = TaxGroup()
        tax_group = resp['tax_group']
        tax_group_obj.set_tax_group_id(tax_group['tax_group_id'])
        tax_group_obj.set_tax_group_name(tax_group['tax_group_name'])
        tax_group_obj.set_tax_group_percentage(tax_group[\
        'tax_group_percentage'])
        for value in tax_group['taxes']:
            tax = Tax()
            tax.set_tax_id(value['tax_id'])
            tax.set_tax_name(value['tax_name'])
            tax.set_tax_percentage(value['tax_percentage'])
            tax.set_tax_type(value['tax_type'])
            tax_group_obj.set_taxes(tax)
        return tax_group_obj

    def get_opening_balance(self, resp):
        """This method parses the given response and returns opening balance 
            object.

        Args:
            resp(dict): Response containing json object for opening balance.

        Returns:
            instance: Opening balance object.

        """
        opening_balance_obj = OpeningBalance()
        opening_balance = resp['opening_balance'] 
        opening_balance_obj.set_opening_balance_id(opening_balance[\
        'opening_balance_id'])
        opening_balance_obj.set_date(opening_balance['date'])
        for value in opening_balance['accounts']:
            accounts = Account()
            accounts.set_account_split_id(value['account_split_id'])
            accounts.set_account_id(value['account_id'])
            accounts.set_account_name(value['account_name'])
            accounts.set_debit_or_credit(value['debit_or_credit'])
            accounts.set_exchange_rate(value['exchange_rate'])
            accounts.set_currency_id(value['currency_id'])
            accounts.set_currency_code(value['currency_code'])
            accounts.set_bcy_amount(value['bcy_amount'])
            accounts.set_amount(value['amount'])
            opening_balance_obj.set_accounts(accounts)
        return opening_balance_obj

    def get_autoreminders(self, resp):
        """This method parses the given response and returns autoreminders list.
        
        Args:
            resp(dict): Response containing json object for autoreminders.

        Returns:
            instance: Reminder list object.

        """
        autoreminders = AutoReminderList()
        for value in resp['autoreminders']:
            autoreminders_obj = Autoreminder()
            autoreminders_obj.set_autoreminder_id(value['autoreminder_id'])
            autoreminders_obj.set_is_enabled(value['is_enabled'])
            autoreminders_obj.set_notification_type(value['type'])
            autoreminders_obj.set_address_type(value['address_type'])
            autoreminders_obj.set_number_of_days(value['number_of_days'])
            autoreminders_obj.set_subject(value['subject'])
            autoreminders_obj.set_body(value['body'])
            autoreminders_obj.set_order(value['order'])
            autoreminders.set_auto_reminders(autoreminders_obj)
        return autoreminders

    def get_autoreminder(self, resp):
        """Get auto reminder.

        Args:
            resp(dict): Response containing json object for auto reminders 
                object.

        Returns:
            instance: Auto reminders object.

        """
        autoreminder = resp['autoreminder']
        autoreminder_obj = Autoreminder()
        autoreminder_obj.set_autoreminder_id(autoreminder['autoreminder_id'])
        autoreminder_obj.set_is_enabled(autoreminder['is_enabled'])
        autoreminder_obj.set_notification_type(autoreminder['type'])
        autoreminder_obj.set_address_type(autoreminder['address_type'])
        autoreminder_obj.set_number_of_days(autoreminder['number_of_days'])
        autoreminder_obj.set_subject(autoreminder['subject'])
        autoreminder_obj.set_body(autoreminder['body'])
        placeholders_obj = PlaceHolder()
        placeholders = resp['placeholders'] 
        for value in placeholders['Invoice']: 
            invoice = Invoice()
            invoice.set_name(value['name'])
            invoice.set_value(value['value'])
            placeholders_obj.set_invoice(invoice)
        for value in placeholders['Customer']:
            customer = Customer()
            customer.set_name(value['name'])
            customer.set_value(value['value'])
            placeholders_obj.set_customer(customer)
        for value in placeholders['Organization']:
            organization = Organization()
            organization.set_value(value['value'])
            organization.set_name(value['name'])
            placeholders_obj.set_organization(organization)
        autoreminder_obj.set_placeholders(placeholders)
        return autoreminder_obj

    def get_manual_reminders(self, resp):
        """This method parses the given response and returns manual reminders 
            list.

        Args:
            resp(dict): Response containing json object for manual reminders 
                list.

        Returns:
            list of instance: List of manual reminders object.

        """
        manual_reminders = ManualReminderList()
        for value in resp['manualreminders']:
            manual_reminder = ManualReminder()       
            manual_reminder.set_manualreminder_id(value['manualreminder_id'])
            manual_reminder.set_type(value['type'])
            manual_reminder.set_subject(value['subject'])
            manual_reminder.set_body(value['body'])
            manual_reminder.set_cc_me(value['cc_me'])
            manual_reminders.set_manual_reminders(manual_reminder)
        return manual_reminders

    def get_manual_reminder(self, resp):
        """Get manual reminder.

        Args:
            resp(dict): Response containing json object for manual reminder.

        Returns:
            instance: Manual reminders object.

        """
        manualreminder = resp['manualreminder']
        manualreminder_obj = ManualReminder()
        manualreminder_obj.set_manualreminder_id(manualreminder[\
        'manualreminder_id'])
        manualreminder_obj.set_type(manualreminder['type'])
        manualreminder_obj.set_subject(manualreminder['subject'])
        manualreminder_obj.set_body(manualreminder['body'])
        manualreminder_obj.set_cc_me(manualreminder['cc_me'])
        placeholders = resp['placeholders']
        placeholders_obj = PlaceHolder()
        for value in placeholders['Invoice']:
            invoice = Invoice()
            invoice.set_name(value['name'])
            invoice.set_value(value['value'])
            placeholders_obj.set_invoice(invoice)
        for value in placeholders['Customer']:
            customer = Customer()
            customer.set_name(value['name'])
            customer.set_value(value['value'])
            placeholders_obj.set_customer(customer)
        for value in placeholders['Organization']:
            organization = Organization()
            organization.set_name(value['name'])
            organization.set_value(value['value'])
            placeholders_obj.set_organization(organization)
        manualreminder_obj.set_placeholders(placeholders_obj)
        return manualreminder_obj
            

        

        

        
 



#$Id$

class Email:
    """This class is used to create a Email object."""
    def __init__(self):
        """Initialize the parameters for email object."""
        self.to_mail_ids = []
        self.cc_mail_ids = []
        self.subject = ''
        self.body = ''
        self.send_from_org_email_id = None
        self.file_name = ''
        self.contact_id = ''
        self.customer_id = ''
        self.gateways_configured = None
        self.attachment_name = ''
        self.to_contacts = [] 
        self.from_emails = []
        self.error_list = []
        self.email_templates = []
        self.deprecated_placeholders_used = []
        self.gateways_associated = None
        self.attach_pdf = None
        self.file_name_without_extension = None

    def set_to_mail_ids(self, to_mail_ids):
        """Set the list of email address of the recipients.
         
        Args:
            to_mail_ids(list): List of email address of the recipients.
        
        """
        self.to_mail_ids.extend(to_mail_ids)
 
    def get_to_mail_ids(self):
        """Get the list of email address of the recipients.
         
        Returns:
            list: List of email address of recipients.
 
        """
        return self.to_mail_ids

    def set_cc_mail_ids(self, cc_mail_ids):
        """Set the list of email address of the recipients to be cced.
  
        Args:
            cc_mail_ids(list): List of email address of the recipients to be 
                cced.
 
        """
        self.cc_mail_ids.extend(cc_mail_ids)
 
    def get_cc_mail_ids(self): 
        """Get the list of email address of the recipients to be cced.
     
        Returns:
            list: List of email address of the recipients to be cced.

        """ 
        return self.cc_mail_ids

    def set_subject(self, subject):
        """Set subject for the email.
     
        Args:
            subject(str): Subject for the email.
 
        """
        self.subject = subject
 
    def get_subject(self): 
        """Get subject of the email.
       
        Returns:
            str: Subject of the email.
        
        """  
        return self.subject

    def set_body(self, body):
        """Set body for the email.
      
        Args:
            body(str): Body for the mail.
      
        """
        self.body = body
   
    def get_body(self):
        """Get body of the mail.
       
        Returns:
            str: Body of the mail.
 
        """
        return self.body

    def set_send_from_org_email_id(self, send_from_org_email_id):
        """Set the boolean to trigger the email from the organization's 
            email address.
        
        Args:
            send_from_org_email_id(bool): True to send email from the 
                organization's email address else False.
     
        """
        self.send_from_org_email_id = send_from_email_id
 
    def get_send_from_org_email_id(self):
        """Get whether to send email from organization's email adress or not.
        
        Returns:
            bool: True send email from organization's email address else False.
 
        """
        return self.send_from_org_email_id
 
    def set_file_name(self, file_name):
        """Set file name that is attached with email.
 
        Args:
            file_name(str): Name of the file that is attached with mail.

        """
        self.file_name = file_name

    def get_file_name(self):
        """Get file name that is attached with email.
 
        Returns: 
            str: Name of the file that is attached.
 
        """
        return self.file_name

    def set_contact_id(self, contact_id):
        """Set contact id.
         
        Args:
            contact_id(str): Contact id.
    
        """
        self.contact_id = contact_id
 
    def get_contact_id(self):
        """Get contact id.
         
        Returns:
            str: Contact id.
    
        """
        return self.contact_id

    def set_customer_id(self, customer_id):
        """Set customer id.
        
        Args:
            customer_id(str): Customer id.
 
        """
        self.customer_id = customer_id

    def get_customer_id(self):
        """Get customer id.
         
        Returns:
            str: Customer id.

        """
        return self.customer_id
  
    def set_gateways_configured(self, gateways_configured):
        """Set gateways configured.
         
        Args:
            gateways_configured(str): Gateways configured.
        
        """  
        self.gateways_configured = gateways_configured

    def get_gateways_configure(self):
        """Get gateways configured.
        
        Returns:
            str: Gateways configured.
       
        """
        return self.gateways_configured
  
    def set_attachment_name(self, attachment_name):
        """Set attachment name.
        
        Args:
            attachment_name(str): Attachment name.
        
        """
        self.attachment_name = attachment_name

    def get_attachment_name(self):
        """Get attachment name.
        
        Returns:
            str: Attachment name.
        
        """
        return self.attachment_name

    def set_to_contacts(self, to_contacts):
        """Set the details for the contacts for which mail has to be sent.
     
        Args:
            to_contacts(instance): To_contacts object.
 
        """
        self.to_contacts.append(to_contacts)
 
    def get_to_contacts(self):
        """Get the details of the contacts for which mail has to be sent.
        
        Returns:
            list: List of to_contacts object.
 
        """
        return self.to_contacts

    def set_from_emails(self, from_emails):
        """Set from email details.
 
        Args:
            from_emails(instance): From emails object.
       
        """ 
        self.from_emails.append(from_emails)

    def get_from_emails(self):
        """Get from email details.
        
        Returns:
            list: lList of from_emails object.

        """
        return self.from_emails

    def set_error_list(self, error_list):
        """Set error list.
       
        Args:
            error_list(list): List of errors.
      
        """
        self.error_list.extend(error_list)
 
    def get_error_list(self):
        """Get error list.
        
        Returns:
            list: List of errors.
         
        """
        return self.error_list
   
    def set_email_templates(self, email_template):  
        """Set email templates.
      
        Args: 
            email_template(instance): Email templates object.

        """
        self.email_templates.append(email_template)
 
    def get_email_templates(self):
        """Get email templates.
          
        Returns:
            list: List of email templates object.
        
        """
        return self.email_templates

    def set_deprecated_placeholders_used(self, deprecated_placeholders_used):
        """Set deprecated placeholders used.
         
        Args:
            set_deprecated_placeholders_used(list): List of deprecated 
                placeholders.
        
        """
        self.deprecated_placeholders_used.extend(deprecated_placeholders_used)

    def get_deprecated_placeholders_used(self):
        """Get deprecated placeholders used.
       
        Returns:
            list: List of deprecated placeholders.
 
        """
        return self.deprecated_placeholders_used

    def set_gateways_associated(self, gateways_associated):
        """Set whether gateways are associated or not.
         
        Args:
            gateways_associated(bool): True if gateways are associated else 
                False.

        """
        self.set_gateways_associated = gateways_associated

    def get_gateways_associated(self):
        """Get whether gateways are associated or not.
 
        Returns:
            bool: True if gateways are associated else False.

        """
        return self.gateways_associated

    def set_attach_pdf(self, attach_pdf):
        """Set whether pdf is attached or not.
 
        Args:
            attach_pdf(bool): True if pdf is attached else False.
 
        """
        self.attach_pdf = attach_pdf
 
    def get_attach_pdf(self):
        """Get whether pdf is attached or not.
        
        Returns:
            bool: True if pdf is attached else False.
 
        """
        return self.attach_pdf
   
    def set_file_name_without_extension(self, file_name_without_extension):
        """Set file name without extension.
         
        Args:
            file_name_without_extension(str): File name without extension.
       
        """
        self.file_name_without_extension = file_name_without_extension

    def get_file_name_without_extension(self):
        """Get filename without extension.
         
        Returns:
            str: File name without extension.
 
        """  
        return self.file_name_without_extension

    def to_json(self):
        """This method is used to convert email object to Json object.

        Returns:
            dict: Dictionary containing json object for email object.

        """
        email = {}
        email['subject'] = self.subject
        email['body'] = self.body
        email['to_mail_ids'] = self.to_mail_ids
        if not self.cc_mail_ids:
            email['cc_mail_ids'] = self.cc_mail_ids
        if self.send_from_org_email_id:
            email['send_from_org_email_id'] = self.sed_from_org_email_id
        return email
  




#$Id$

class AddressFormat: 
    """This class is used to create object for Address Formats."""
    def __init__(self): 
        """Initialize parameters for address formats."""
        self.organization_address_format = ''
        self.customer_address_format = ''

    def set_organization_address_format(self, organization_address_format):
        """Set organization address format.

        Args: 
            organization_address_format(str): Organization address format.

        """
        self.organization_address_format = organization_address_format

    def get_organization_address_foramt(self):
        """Get organization address format.

        Returns:
            str: Organization address format.

        """
        return self.organization_address_format

    def set_customer_address_format(self, customer_address_format):
        """Set customer address format.

        Args:
            customer_address_format(str): Customer address format.

        """
        self.customer_address_format = customer_address_format

    def get_customer_address_format(self):
        """Get customer address format.

        Returns:
            str: Customer address format.

        """
        return self.customer_address_format



#$Id$

from books.model.PageContext import PageContext

class BaseCurrencyAdjustmentList:
    """This class is used to creste object for Base Currency Adjustment List."""
    def __init__(self):
        """Initialize parameters for Base Currency list object."""
        self.base_currency_adjustments = []
        self.page_context = PageContext()

    def set_base_currency_adjustments(self, base_currency_adjustment):
        """Set base currency adjustment.

        Args:
            base_currency_adjustment(instance): Base currency adjustment object.

        """
        self.base_currency_adjustments.append(base_currency_adjustment)
 
    def get_base_currency_adjustments(self): 
        """Get base currency adjustments.

        Returns:
            list of instance: List of base currency adjustments.

        """
        return self.base_currency_adjustments

    def set_page_context(self, page_context):
        """Set page context.

        Args:
            page_context(instance): Page context object.

        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page context.

        Returns:
            instance: Page context.

        """
        return self.page_context




#$Id$

class BillPayment:
    """This class is used to create object for payments."""
    def __init__(self):
        """Initialize parameters for Payments object."""
        self.payment_id = ''
        self.bill_id = ''
        self.bill_payment_id = ''
        self.payment_mode = ''
        self.description = ''
        self.date = ''
        self.reference_number = ''
        self.exchange_rate = 0.0
        self.amount = 0.0
        self.paid_through_account_id = ''
        self.paid_through_account_name = ''
        self.is_single_bill_payment = None
        self.amount_applied = 0.0
        self.vendor_id = ''
        self.vendor_name = ''
        self.paid_through = ""
      
    def set_paid_through(self, paid_through):
        """Set paid through.

        Args:
            paid_through(str): Paid through.

        """
        self.paid_through = paid_through

    def get_paid_through(self):
        """Get paid through.

        Returns:
            str: Paid through.

        """
        return self.paid_through
    
    def set_vendor_name(self, vendor_name):
        """Set vendor name.

        Args:
            vendor_name(str): Vendor name.
 
        """
        self.vendor_name = vendor_name

    def get_vendor_name(self):
        """Get vendor name.

        Returns:
            str: Vendor name.
 
        """
        return self.vendor_name

    def set_vendor_id(self, vendor_id):
        """Set vendor id.

        Args:
            vendor_id(str): Vendor id.

        """
        self.vendor_id = vendor_id
     
    def get_vendor_id(self):
        """Get vendor id.

        Returns:
            str: Vendor id.

        """
        return self.vendor_id

    def set_amount_applied(self, amount_applied):
        """Set amount applied.

        Args:
            amount_applied(float): Amount applied.

        """
        self.amount_applied = amount_applied

    def get_amount_applied(self):
        """Get amount applied.

        Returns:
            float: Amount applied.

        """
        return self.amount_applied
  
    def set_payment_id(self, payment_id):
        """Set payment id.

        Args:
            payment_id(str): Payment id.

        """
        self.payment_id = payment_id

    def get_payment_id(self): 
        """Get payment id.

        Returns:
            str: Payment id.

        """
        return self.payment_id
  
    def set_bill_id(self, bill_id):
        """Set bill id.

        Args:
            bill_id(str): Bill id.

        """
        self.bill_id = bill_id

    def get_bill_id(self):
        """Get bill id.

        Returns:
            str: Bill id.

        """
        return bill_id
  
    def set_bill_payment_id(self, bill_payment_id):
        """Set bill payment id.

        Args:
            bill_payment_id(str): Bill payment id.

        """
        self.bill_payment_id = bill_payment_id

    def get_bill_payment_id(self):
        """Get bill payment id.

        Returns:
            str: Bill payment id.

        """
        return self.bill_payment_id
  
    def set_payment_mode(self, payment_mode):
        """Set payment id.

        Args:
            payment_mode(str): Payment mode.

        """
        self.payment_mode = payment_mode

    def get_payment_mode(self):
        """Get payment mode.

        Returns:
            str: Payment mode.

        """
        return self.payment_mode

    def set_description(self, description):
        """Set description.
 
        Args:
            description(str): Description.

        """
        self.description = description

    def get_description(self):
        """Get description.

        Returns:
            str: Description.

        """
        return self.description

    def set_date(self, date):
        """Set date.

        Args:
            date(str): Date.

        """
        self.date = date
 
    def get_date(self):
        """Get date.

        Returns:
            str: Date.

        """
        return self.date
 
    def set_reference_number(self, reference_number):
        """Set reference number.

        Args:
            reerence_number(str): Refference number.

        """
        self.reference_number = reference_number

    def get_reference_number(self):  
        """Get reference number.

        Returns:
            str: Reference number.

        """
        return self.reference_number

    def set_exchange_rate(self, exchange_rate):
        """Set exchange rate.

        Args:
            exchange_rate(float): Exchange rate.
 
        """
        self.exchange_rate = exchange_rate

    def get_exchange_rate(self):
        """Get exchange rate.
 
        Returns:
            float: Exchange rate.

        """
        return self.exchange_rate

    def set_amount(self, amount):
        """Set amount.

        Args:
            amount(float): Amount.

        """
        self.amount = amount

    def get_amount(self):
        """Get amount.

        Returns:
            float: Amount.

        """
        return self.amount

    def set_paid_through_account_id(self, paid_through_account_id):
        """Set paid through account id.

        Args:
            paid_through_account_id(str): Paid through account id.

        """
        self.paid_through_account_id = paid_through_account_id
 
    def get_paid_through_account_id(self):
        """Get paid through account id.

        Returns:
            str: Paid through account id.

        """
        return self.paid_through_account_id

    def set_paid_through_account_name(self, paid_through_account_name):
        """Set paid through account name.

        Args:
            paid_through_account_name(str): Paid through account name.

        """
        self.paid_through_account_name = paid_through_account_name

    def get_paid_through_account_name(self, paid_through_acount_name):
        """Get paid through account name.

        Returns:
            str: Paid through account name.

        """
        return self.paid_through_account_name

    def set_is_single_bill_payment(self, is_single_bill_payment):
        """Set whether it is single payment bill.

        Args:
            is_single_bill_payment(bool): True if it is single bill payment.

        """
        self.is_single_bill_payment = is_single_bill_payment

    def get_is_single_bill_payment(self):
        """Get whether it is single payment bill.

        Returns:
            bool: True if it is single bill payment.

        """
        return self.is_single_bill_payment






#$Id$

class EmailHistory:
    """This class is used to create an object for Email history."""
    def __init__(self):
        """Initialize the parameters for Email hstory object."""
        self.mailhistory_id = ''
        self.from_mail_id = ''
        self.to_mail_ids = ''
        self.subject = ''
        self.date = ''
        self.type = 0

    def set_mailhistory_id(self, mailhistory_id):
        """Set mail history id.

        Args:
            mailhistory_id(str): Mail history id.

        """
        self.mailhistory_id = mailhistory_id

    def get_mailhistory_id(self):
        """Get mail history id.

        Returns:
            str: Mail history id.

        """
        return self.mailhistory_id

    def set_from(self, from_mail_id):
        """Set from mail id.

        Args:
            from_mail_id(str): From mail id.

        """
        self.from_mail_id = from_mail_id

    def get_from(self):
        """Get from mail id.

        Returns:
            str: From mail id.

        """
        return self.from_mail_id

    def set_to_mail_ids(self, to_mail_ids):
        """Set to mail id.

        Args:
            to_mail_ids(str): To mail ids.

        """
        self.to_mail_ids = to_mail_ids

    def get_to_mail_ids(self):
        """Get to mail ids.

        Returns: 
            str: To mail ids.

        """
        return self.to_mail_ids

    def set_subject(self, subject):
        """Set subject.

        Args: 
            subjecct(str): Subject.

        """
        self.subject = subject

    def get_subject(self):
        """Get subject.

        Returns:
            str: Subject.

        """
        return self.subject

    def set_date(self, date):
        """Set date.

        Args:
            date(str): Date.

        """
        self.date = date

    def get_date(self):
        """Get date.

        Returns:
            str: Date.

        """
        return self.date

    def set_type(self, email_type):
        """Set type.

        Args:
            type(int): Type.

        """
        self.type = email_type

    def get_type(self):
        """Get type.
 
        Returns:
            int: Type.
 
        """
        return self.type





#$Id$

class OpeningBalance:
    """This class is used to create object for opening balance."""
    def __init__(self):
        """Initialize parameters for opening balance."""
        self.opening_balance_id = ''
        self.date = ''
        self.accounts = []
        
    def set_opening_balance_id(self, opening_balance_id):
        """Set opening balance id.

        Args:
            opening_balance_id(str): Opening balance id.
 
        """
        self.opening_balance_id = opening_balance_id

    def get_opening_balance_id(self):
        """Get opening balance id.

        Returns:
            str: Opening balancce id.

        """
        return self.opening_balance_id

    def set_date(self, date):
        """Set date.

        Args:
            date(str): Date.

        """
        self.date = date

    def get_date(self):
        """Get date.

        Returns:
            str: Date.

        """
        return self.date

    def set_accounts(self, account):
        """Set accounts.

        Args:
            account(instance): Account object.

        """
        self.accounts.append(account)

    def get_accounts(self):
        """Get accounts.

        Returns:
            list of instance: List of accounts object.

        """
        return self.accounts
        
    def to_json(self):
        """This method is used to convert opening balance object to json object.

        Returns:
            dict: Dictionary containing json object for opening balance.
         
        """
        data = {}
        if self.date != '':
            data['date'] = self.date
        if self.accounts:
            data['accounts'] = []
            for value in self.accounts:
                account = value.to_json()
                data['accounts'].append(account)
        return data



#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.JournalsParser import JournalsParser
from books.api.Api import Api
from json import dumps

base_url = Api().base_url + 'journals/'
zoho_http_client = ZohoHttpClient()
parser = JournalsParser()

class JournalsApi:
    """Journals Api class is used to:
     
    1.Get journals list.
    2.Get the details of the journal.
    3.Create a journal.
    4.Updates the journal with given information.
    5.Deletes the given journal.

    """
    def __init__(self, authtoken, organization_id):
        """Initialize parameters for Journals Api.

        Args:
            authtoken(str): User's authtoken.
            organization_id(str): User's organization id.

        """        
        self.details = {
            'authtoken': authtoken,
            'organization_id': organization_id
            }
  
    def get_journals(self, parameter=None):
        """Get Journals list.
 
        Args:
            parameter(dict): Filter with which the list has to be displayed.

        Returns:
            instance: Journals List object.

        """    
        resp = zoho_http_client.get(base_url, self.details, parameter)
        return parser.get_list(resp)

    def get(self, journal_id):
        """Get details of a journal.

        Args:
            journal_id(str): Journal id.
 
        Returns:
            instance: Journals object.

        """  
        url = base_url + journal_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_journal(resp) 
  
    def create(self, journal):
        """Create a journal.
 
        Args:
            journal(instance): Journal object. 
 
        Returns:
            instance: Journal object.

        """     
        json_object = dumps(journal.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.post(base_url, self.details, data)
        return parser.get_journal(resp) 

    def update(self, journal_id, journal):
        """Update an existing journal.

        Args:
            journal_id(str): Journal id.
            journal(instance): Journal object.

        Returns:
            instance: Journal object.

        """
        url = base_url + journal_id
        json_object = dumps(journal.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_journal(resp) 

    def delete(self, journal_id):
        """Delete the given journal.

        Args:
            journal_id(str): Journal id.

        """
        url = base_url + journal_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 




#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.BankTransactionsParser import BankTransactionsParser
from books.api.Api import Api
from os.path import basename
from json import dumps

base_url = Api().base_url + 'banktransactions/'
zoho_http_client = ZohoHttpClient()
parser = BankTransactionsParser()

class BankTransactionsApi:
    """Bank Transactions Api class is used to 

    1.Get all the transaction details involved in an account.
    2.Get the details of a transaction.
    3.Create a bank transaction based on allowed transaction types.
    4.Update an existing bank transaction.
    5.Delete a transaction from an account.
    6.Get matching transactions.
    7.Match an uncategorized transaction with an existing transaction in the 
      account.
    8.Unmatch a transaction that was previously matched and make it 
      uncategorized.
    9.Get a list of all the associated that were matched or categorized to the 
      given imported transaction.
    10.Exclude a transaction from your bank or credit card account.
    11.Restore an excluded transaction in your account.
    12.Categorize an uncategorized transaction by creating a new transaction.
    13.Categorize an uncategorized transaction as a refund from credit note.
    14.Categorize an uncategorized transaction as vendor payment.
    15.Categorize an uncategorized transaction as customer payment.
    16.categorize an uncategorized transaction as Expense.
    17.Revert a categorized transaction as uncategorized.

    """ 
    def __init__(self, authtoken, organization_id):
        """Initialize Bank transaction api using user's authtoken and 
            organization id.

        Args: 
            authotoken(str): User's Authtoken.
            organization id(str): User's Organization id.

        """
        self.details = { 
            'authtoken': authtoken, 
            'organization_id': organization_id
            }
  
    def get_bank_transactions(self, parameter=None): 
        """Get all transaction details involved in an account.

        Args:
            parameter: Filter with which the list has to be displayed.

        Returns:
            instance: Bank transaction list object.
   
        """
        resp = zoho_http_client.get(base_url, self.details, parameter)
        return parser.get_list(resp)

    def get(self, transaction_id):
        """Get bank transaction details.

        Args:
            transaction_id(str): Transaction id.

        Returns:
            instance: Bank transaction object.

        """
        url = base_url + transaction_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_transaction(resp) 
 
    def create(self, transaction):
        """Create a bank transaction for an account.

        Args:
            transaction(instance): Bank Transaction object.

        Returns:
            instance: Bank transaction object.

        """
        json_object = dumps(transaction.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.post(base_url, self.details, data)
        return parser.get_transaction(resp) 

    def update(self, bank_transaction_id, transaction):
        """Update an existing transaction.

        Args:
            bank_transaction_id(str): Bank transaction id.
            transaction(instance): Bank transaction object.

        Returns: 
            instance: Bank transaction object.

        """   
        url = base_url + bank_transaction_id
        json_object = dumps(transaction.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_transaction(resp) 

    def delete(self, transaction_id): 
        """Delete an existing transaction.

        Args:
            transaction_id(str): Transaction id.

        Returns: 
            str: Success message('The transaction has been deleted.').
 
        """
        url = base_url + transaction_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 
  
    def get_matching_transactions(self, bank_transaction_id, parameters=None): 
        """Provide criteria to search for matching uncategorized transactions.

        Args:
            bank_transaction_id(str): Bank transaction id.
            parameters(dict, optional): Filter with which matching transactions
             has to be displayed.

        Returns: 
            instance: Transactions list object.

        """
        url = base_url + 'uncategorized/' + bank_transaction_id + \
              '/match'
        resp = zoho_http_client.get(url, self.details, parameters) 
        return parser.get_matching_transaction(resp) 
  
    def match_a_transaction(self, transaction_id, transactions, 
                            account_id=None):
        """Match an uncategorized transaction with an existing transaction in 
            the account.
 
        Args:
            transaction_id(str): Transaction id.
            transactions(list of instance): List of transactions object.
            account_id(str): Account id.
 
        Returns:
            str: Success message('The Uncategorized transaction is linked to 
                 the selected transaction(s) in Zoho Books.').
 
        """
        url = base_url + 'uncategorized/' + transaction_id + '/match'
        data = {
            'transactions_to_be_matched': []
            }
        for value in transactions:
            transactions_to_be_matched = {}
            transactions_to_be_matched['transaction_id'] = \
            value.get_transaction_id()
            transactions_to_be_matched['transaction_type'] = \
            value.get_transaction_type()
            data['transactions_to_be_matched'].append(\
            transactions_to_be_matched) 
        if account_id is None: 
            query = None
        else:
             query = {
                'account_id': account_id
                }
        json_string = {
            'JSONString': dumps(data)
            }
        resp = zoho_http_client.post(url, self.details, json_string, query)
        return parser.get_message(resp) 

    def unmatch_a_matched_transaction(self, transaction_id, account_id=None):
        """Unmatch a transaction that was previously matched and make it 
            uncategorized.
 
        Args:
            transaction_id(str): Transaction id.
            account_id(str): Account id.

        Returns:
            str: Success message('The transaction has been unmatched.').
 
        """
        url = base_url + transaction_id + '/unmatch'
        if account_id is None:
            parameter = None
        else:
            parameter = {
                'account_id': account_id
                }
        data = {
            'JSONString': ''
            }
        resp = zoho_http_client.post(url, self.details, data, parameter)
        return parser.get_message(resp) 

    def get_associated_transactions(self, transaction_id, sort_column=None):
        """Get a list of all the transactions that were matched or categorized 
            to the given imported transaction.
   
        Args:
            transaction_id(str): Transaction id.
            sort_column(str): Param description. Allowed values are statement 
                date. Defaults to None.
         
        Returns:
            instance: Transaction list object.

        """ 
        url = base_url + transaction_id + '/associated'
        if sort_column is None:
            param = None
        else:
            param = {
                'sort_column': sort_column
                }
        resp = zoho_http_client.get(url, self.details, param) 
        return parser.get_associated_transaction(resp) 

    def exclude_a_transaction(self, transaction_id, account_id=None):
        """Exclude a transaction from your bank or credit card account.

        Args:
            transaction_id(str): Transaction id.
            account_id(str, optional): Account id.

        Returns:
            str:  Success message('The transaction has been excluded.').

        """
        url = base_url + 'uncategorized/' + transaction_id + '/exclude'
        if account_id is None:
            param = None
        else:
            param = {
                'account_id': account_id
                }
        data = {
            'JSONString': ''
            }
        resp = zoho_http_client.post(url, self.details, data, param)
        return parser.get_message(resp) 

    def restore_a_transaction(self, transaction_id, account_id=None):
        """Restore a transaction.

        Args:
            transaction_id(str): Transaction id.
            account_id(str,optional): Account id.

        Returns:
            str: Success message('The excluded transactions has been 
                 restored.').

        Raises: 
            Books Exception: If status is not '200' or '201'.

        """
        url = base_url + 'uncategorized/' + transaction_id + '/restore'
        if account_id is None:
            param = None
        else:
            param = {
                'account_id': account_id
                }
        data = {
            'JSONString': ''
            }
        resp = zoho_http_client.post(url, self.details, data, param)
        return parser.get_message(resp) 

    def categorize_an_uncategorized_transaction(self, transaction_id, \
        transaction):
        """Categorize an uncategorized transaction by creating a new 
            transaction.

        Args: 
            transaction_id(str): Transaction id.
            transaction(instance): Bank transaction object.

        Returns:
            str: Success message('The transaction is now categorized.').

        """
        url = base_url + 'uncategorized/' + transaction_id + '/categorize'
        json_object = dumps(transaction.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_message(resp) 

    def categorize_as_credit_note_refunds(self, transaction_id, credit_note):
        """Categorize an uncategorized transaction as a refund from a credit 
            note.

        Args:
            transaction_id(str): Transaction id.
            credit_note(instance): Credit note object.

        Returns:
            str: Success message('The transaction is now categorized.').

        """
        url = base_url + 'uncategorized/' + transaction_id + \
              '/categorize/creditnoterefunds'
        json_object = dumps(credit_note.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_message(resp)

    def categorize_as_vendor_payment(self, transaction_id, vendor_payment):
        """Categorize an uncategorized transaction as Vendor payment.

        Args:
            transaction_id(str): Transaction id.
            vendor_payment(instance): Vendor payment object.

        Returns:
            str: Success message('The transaction is now categorized.').

        """  
        url = base_url + 'uncategorized/' + transaction_id + \
              '/categorize/vendorpayments'
        json_object = dumps(vendor_payment.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_message(resp) 
  
    def categorize_as_customer_payment(self, transaction_id, customer_payment, \
        contact_ids=None):
        """Categorize an uncategorized transaction as Customer Payment.

        Args:
            transaction_id(str): Transaction id.
            customer_payment(instance): Customer payment object.
            contact_ids(str, optional): Contact ids. Defaults to None.

        Returns:
            str: Success message('The transaction is now categorized.').

        """
        url = base_url + 'uncategorized/' + transaction_id + \
              '/categorize/customerpayments'
        json_object = dumps(customer_payment.to_json())
        data = {
            'JSONString': json_object
            }
        if contact_ids is None:
            param = None
        else:
            param = {
                'contact_ids': contact_ids
                }
        resp = zoho_http_client.post(url, self.details, data, param)
        return parser.get_message(resp) 

    def categorize_as_expense(self, transaction_id, expense, receipt=None):
        """Categorize an uncategorized transaction as expense.

        Args:
            transaction_id(str): Transaction id.
            expense(instance): Expense object.
            receipt(file, optional): File to be attached. Allowed Extensions 
                are gif, png, jpeg, jpg, bmp and pdf.

        Returns:
            str: Success message('The transaction is now categorized.').

        """ 
        url = base_url + 'uncategorized/' + transaction_id + \
              '/categorize/expense'  
        json_object = dumps(expense.to_json())
        data = {
            'JSONString': json_object
            }
        if receipt is None:
            attachments = None
        else:
            attachments = [{
                'receipt': {
                    'filename': basename(receipt), 
                    'content': open(receipt).read()
                    } 
                }]
        print(data)
        resp = zoho_http_client.post(url, self.details, data, None, attachments)
        return parser.get_message(resp) 

    def uncategorize_a_categorized_transaction(self, transaction_id, 
                                               account_id=None):
        """Revert a categorized transaction as uncategorized.

        Args:
            transaction_id(str): Transaction id.
            account_id(str, optional): Account id. Defaults to None.

        Returns:
            str: Success message('The transaction has been uncategorized.').

        """  
        url = base_url + transaction_id + '/uncategorize'
        if account_id is None:
            query = None
        else:
            query = {
                'account_id': account_id
                }
        data = {
            'JSONString': ''
            }
        resp = zoho_http_client.post(url, self.details, data,  query)
        return parser.get_message(resp) 




#$Id$

class EstimateSetting:
    """This class is used to create object for estimate settings."""
    def __init__(self):
        """Initialize parameters for estimate settings."""
        self.auto_generate = None
        self.prefix_string = ""
        self.start_at = 0
        self.next_number = ""
        self.quantity_precision = 0
        self.discount_type = ""
        self.is_discount_before_tax = None
        self.reference_text = ""
        self.notes = ""
        self.terms = ""
        self.terms_to_invoice = None
        self.notes_to_invoice = None
        self.warn_estimate_to_invoice = None
        self.is_sales_person_required = None
        self.default_template_id = ""
    
    def set_default_template_id(self, default_template_id): 
        """Set default template id.
       
        Args:
            default_template_id(str): Default template id.
 
        """
        self.default_template_id = default_template_id

    def get_default_template_id(self):
        """Get default template id.

        Returns: 
            str: Default template id.

        """ 
        return self.default_template_id       
 
    def set_auto_generate(self, auto_generate):
        """Set whether to enable or disable auto number generation.
 
        Args:
            auto_generate(bool): True to enable auto number generation.
  
        """
        self.auto_generate = auto_generate

    def get_auto_generate(self):
        """Get whether to enable or disable auto number generation.

        Returns:
            bool: True to enable auto number generation.

        """
        return self.auto_generate

    def set_prefix_string(self, prefix_string):
        """Set prefix string.

        Args:
            prefix_string(str): Prefix string.

        """
        self.prefix_string = prefix_string

    def get_prefix_string(self):
        """Get prefix string.
 
        Returns: 
            str: Prefix string.

        """
        return self.prefix_string

    def set_start_at(self, start_at):
        """Set start at.

        Args:
            start_at(int): Start at.

        """
        self.start_at = start_at

    def get_start_at(self):
        """Get start at.

        Returns:
            str: Get start at.

        """ 
        return self.start_at

    def set_next_number(self, next_number):
        """Set next number.

        Args:
            next_number(str): Next number.

        """
        self.next_number = next_number

    def get_next_number(self):
        """Get next number.

        Returns:
            str: Next number.

        """
        return self.next_number

    def set_quantity_precision(self, quantity_precision):
        """Set quantity precision.

        Args:
            quantity_precision(int): Quantity precision.

        """
        self.quantity_precision = quantity_precision

    def get_quantity_precision(self):
        """Get quantity precision.

        Returns:
            int: Quantity precision.

        """
        return self.quantity_precision

    def set_discount_type(self, discount_type):
        """Set discount type.

        Args:
            discount_type(str): Discount type.
 
        """
        self.discount_type = discount_type

    def get_discount_type(self):
        """Get discount type.

        Returns:
            str: Discount type.

        """
        return self.discount_type

    def set_is_discount_before_tax(self, is_discount_before_tax):
        """Set whether to discount before tax.

        Args:
            is_discount_before_tax(bool): True to discount before tax.

        """
        self.is_discount_before_tax = is_discount_before_tax

    def get_is_discount_before_tax(self):
        """Get whether to discount before tax.

        Returns:
            bool: True to discount before tax else false.

        """
        return self.is_discount_before_tax

    def set_reference_text(self, reference_text):
        """Set reference text.

        Args:
            reference_text(str): Reference text.

        """
        self.reference_text = reference_text

    def get_reference_text(self):
        """Get reference text.

        Returns:
            str: Reference text.

        """
        return self.reference_text

    def set_notes(self, notes):
        """Set notes.

        Args:
            notes(str): Notes.

        """
        self.notes = notes

    def get_notes(self):
        """Get notes.

        Returns:
            str: Notes.

        """
        return self.notes

    def set_terms(self, terms):
        """Set terms.

        Args:
            terms(str): Terms.

        """
        self.terms = terms

    def get_terms(self):
        """Get terms.

        Returns:
            str: Terms.

        """
        return self.terms

    def set_terms_to_invoice(self, terms_to_invoice):
        """Set terms to invoice.

        Args:
            terms_to_invoice(bool): True to determine whether terms and conditions field is to be copied from estimate to invoice else False.

        """
        self.terms_to_invoice = terms_to_invoice

    def get_terms_to_invoice(self):
        """Get terms to invoice.

        Returns: 
            bool: True to determine whether terms and conditions field is to be copied from estimate to invoice else False.

        """
        return self.terms_to_invoice

    def set_notes_to_invoice(self, notes_to_invoice):
        """Set notes to invoice.

        Args:
            notes_to_invoice(bool): True to determine whether customer notes field is to be copied from estimate to invoice.
 
        """
        self.notes_to_invoice = notes_to_invoice

    def get_notes_to_invoice(self):
        """Get notes to invoice.

        Returns:
            bool: True to determine whether customer notes field is to be copied from estimate to invoice.

        """
        return self.notes_to_invoice

    def set_warn_estimate_to_invoice(self, warn_estimate_to_invoice):
        """Set whether to warn while converting from estimate to invoice.

        Args:
            warn_estimate_to_invoice(bool): True to warn while converting from estiamte to invoice.

        """
        self.warn_estimate_to_invoice = warn_estimate_to_invoice

    def get_warn_estimate_to_invoice(self):
        """Get whether to warn while converting form estimate to invoice.

        Returns:
            bool: True to warn while converting from estimate to invoice.

        """
        return self.warn_estimate_to_invoice

    def set_is_sales_person_required(self, is_sales_person_required):
        """Set whether sales person is required.

        Args:
            is_sales_person_required(bool): True if sales person is required else false.

        """
        self.is_sales_person_required = is_sales_person_required

    def get_is_sales_person_required(self):
        """Get whether sales person is required.

        Returns:
            bool: True if sales person is required.

        """
        return self.is_sales_person_required

    def to_json(self):
        """This method is used to convert estimate setting object to json object.

        Returns:
            dict: Dictionary containing json object for estimate setting.

        """
        data = {}
        if self.auto_generate != None:
            data['auto_generate'] = self.auto_generate
        if self.prefix_string != "":
            data['prefix_string'] = self.prefix_string
        if self.start_at > 0:
            data['start_at'] = self.start_at
        if self.next_number != '':
            data['next_number'] = self.next_number
        if self.quantity_precision > 0:
            data['quantity_precision'] = self.quantity_precision
        if self.discount_type != '':
            data['discount_type'] = self.discount_type
        if self.reference_text != '':
            data['reference_text'] = self.reference_text
        if self.default_template_id != '':
            data['default_template_id'] = self.default_template_id
        if self.notes != '':
            data['notes'] = self.notes
        if self.terms != '':
            data['terms'] = self.terms
        if self.terms_to_invoice is not None:
            data['terms_to_invoice'] = self.terms_to_invoice
        if self.notes_to_invoice is not None:
            data['notes_to_invoice'] = self.notes_to_invoice
        if self.warn_estimate_to_invoice is not None: 
            data['warn_estimate_to_invoice'] = self.warn_estimate_to_invoice
        if self.is_sales_person_required is not None:
            data['is_sales_person_required'] = self.is_sales_person_required
        return data



#$Id$

class Account:
    """This class is used to create object for accounts object."""
    def __init__(self):
        """Initialize parameters for Accounts."""
        self.account_id = ''
        self.account_name = ''
        self.bcy_balance = 0.0
        self.bcy_balance_formatted = ''
        self.fcy_balance = 0.0
        self.fcy_balance_formatted = ''
        self.adjusted_balance = 0.0
        self.adjusted_balance_formatted = ''
        self.gain_or_loss = 0.0
        self.gain_or_loss_formatted = ''
        self.gl_specific_type = 0

        self.account_split_id = ''
        self.debit_or_credit = ''
        self.exchange_rate = 0.0
        self.currency_id = ''
        self.currency_code = ''
        self.bcy_amount = 0.0
        self.amount = 0.0

    def set_account_id(self, account_id):
        """Set account id.
     
        Args:
            account_id(str): Account id.

        """
        self.account_id = account_id

    def get_account_id(self):
        """Get account id.

        Returns: 
            str: Account id.

        """
        return self.account_id

    def set_account_name(self, account_name): 
        """Set account name.

        Args:
            account_name(str): Account name.

        """
        self.account_name = account_name

    def get_account_name(self):
        """Get account name.

        Returns:
            str: Account name.

        """
        return self.account_name

    def set_bcy_balance(self, bcy_balance):
        """Set bcy balance.

        Args:
            bcy_balance(float): Bcy balance.

        """
        self.bcy_balance = bcy_balance

    def get_bcy_balance(self):
        """Get bcy balance.

        Returns:
            float: Bcy balance.

        """
        return self.bcy_balance

    def set_bcy_balance_formatted(self, bcy_balance_formatted): 
        """Set bcy balance formatted.

        Args:
            bcy_balance_formatted(str): Bcy balance formatted.

        """
        self.bcy_balance_formatted = bcy_balance_formatted

    def get_bcy_balance_formatted(self):
        """Get bcy balance formatted.

        Returns:
            str: Bcy balance formatted.

        """
        return self.bcy_balance_formatted

    def set_fcy_balance(self, fcy_balance):
        """Set fcy balance.

        Args:
            fcy_balance(float): Fcy balance.

        """
        self.fcy_balance = fcy_balance

    def get_fcy_balance(self):
        """Get fcy balance.

        Returns:
            float: Fcy balance.

        """
        return self.fcy_balance

    def set_fcy_balance_formatted(self, fcy_balance_formatted):
        """Set fcy balance formatted.

        Args:
            fcy_balance_formatted(str): Fcy balance formatted.

        """
        self.fcy_balance_formatted = fcy_balance_formatted

    def get_fcy_balance_formatted(self): 
        """Get fcy balance formatted.

        Returns:
            str: Fcy balance formatted.

        """
        return self.fcy_balance_formatted

    def set_adjsuted_balance(self, adjusted_balance):
        """Set adjusted balance.

        Args:
            adjusted_balance(float): Adjusted balance.

        """
        self.adjusted_balance = adjusted_balance

    def get_adjusted_balance(self): 
        """Get adjusted balance.

        Returns: 
            float: Adjusted balance.

        """
        return self.adjusted_balance

    def set_adjusted_balance(self, adjusted_balance):
        """Set adjusted balance.

        Args:
            adjusted_balance(float): Adjusted balance.

        """
        self.adjusted_balance = adjusted_balance

    def get_adjusted_balance(self):
        """Get adjusted balance.

        Returns:
            float: Adjusted balance.

        """
        return self.adjusted_balance

    def set_adjusted_balance_formatted(self, adjusted_balance_formatted):
        """Set adjusted balance formatted.

        Args:
            adjusted_balance_formatted(str): Adjusted balance formatted.

        """
        self.adjusted_balance_formatted = adjusted_balance_formatted

    def get_adjusted_balance_formatted(self): 
        """Get adjusted balance formatted.

        Returns: 
            str: Adjusted balance formatted.

        """
        return self.adjusted_balance_formatted

    def set_gain_or_loss(self, gain_or_loss): 
        """Set gain or loss.

        Args:
            gain_or_loss(float): Gain or loss.

        """
        self.gain_or_loss = gain_or_loss

    def get_gain_or_loss(self):
        """Get gain or loss.
 
        Returns:
            float: Gain or loss.

        """
        return self.gain_or_loss

    def set_gain_or_loss_formatted(self, gain_or_loss_formatted):
        """Set gain or loss formatted.

        Args:
            gain_or_loss_formatted(str): Gain or loss formatted.

        """
        self.gain_or_loss_formatted = gain_or_loss_formatted

    def get_gain_or_loss_formatted(self):
        """Get gain or loss formatted.

        Returns: 
            str: Gain or loss formatted.

        """
        return self.gain_or_loss_formatted

    def set_gl_specific_type(self, gl_specific_type):
        """Set gl specific type.

        Args:
            gl_specific_type(int): Gl specific type.

        """
        self.gl_specific_type = gl_specific_type

    def get_gl_specific_type(self):
        """Get gl specific type.

        Returns:
            int: Gl specific type.

        """
        return self.gl_specific_type

    def set_account_split_id(self, account_split_id):
        """Set account split id.
 
        Args: 
            account_split_id(str): Account split id.
 
        """
        self.account_split_id = account_split_id

    def get_account_split_id(self):
        """Get account split id.

        Returns:
            str: Account split id.
 
        """
        return self.account_split_id

    def set_debit_or_credit(self, debit_or_credit):
        """Set debit or credit.

        Args:
            debit_or_credit(str): Debit or credit.
 
        """
        self.debit_or_credit = debit_or_credit

    def get_debit_or_credit(self):
        """Get debit or credit.

        Returns:
            str: Debit or credit.

        """  
        return self.debit_or_credit

    def set_exchange_rate(self, exchange_rate):
        """Set exchange rate.

        Args:
            exchange_rate(float): Exchange rate.

        """
        self.exchange_rate = exchange_rate

    def get_exchange_rate(self):
        """Get exchange rate.

        Returns:
            float: Exchange rate.

        """
        return self.exchange_rate

    def set_currency_id(self, currency_id): 
        """Set currency id.
 
        Args:
            str: Currecny id.
  
        """
        self.currency_id = currency_id

    def get_currency_id(self): 
        """Get currency id.

        Returns:
            str: Currency id.

        """
        return self.currency_id

    def set_currency_code(self, currency_code):
        """Set currency code.

        Args:
            currency_code(str): Currency code.

        """
        self.currency_code = currency_code

    def get_currency_code(self):
        """Get currency code.

        Returns:
            str: Currency code.

        """
        return self.currency_code

    def set_bcy_amount(self, bcy_amount):
        """Set bcy amount.

        Args:
            bcy_amount(float): Bcy amount.

        """
        self.bcy_amount = bcy_amount

    def get_bcy_amount(self):
        """Get bcy amount.

        Returns:
            float: Bcy amount.

        """
        return self.bcy_amount

    def set_amount(self, amount):
        """Set amount.

        Args:
            amount(float): Amount.

        """
        self.amount = amount 

    def get_amount(self):
        """Get amount.

        Returns: 
            float: Amount.

        """
        return self.amount

    def to_json(self):
        """This method is used to create json object for accounts.

        Returns:
            dict: Dictionary containing json object for accounts.

        """
        data = {}
        if self.account_id != '':
            data['account_id'] = self.account_id
        if self.debit_or_credit != '':
            data['debit_or_credit'] = self.debit_or_credit
        if self.exchange_rate > 0:
            data['exchange_rate'] = self.exchange_rate
        if self.currency_id != '':
            data['currency_id'] = self.currency_id
        if self.amount > 0: 
            data['amount'] = self.amount
        return data




#$Id$

class CreditnoteSetting:
    """This class is used to create object for creditnotes settings."""
    def __init__(self): 
        """Initialize parameters for creditnotes settings."""
        self.auto_generate = None
        self.prefix_string = ""
        self.reference_text = ""
        self.next_number = ""
        self.notes = ""
        self.terms = ""
 
    def set_auto_generate(self, auto_generate):
        """Set whether auto number generation is enabled.

        Args: 
            auto_generate(bool): True to enable auto number genration else false.

        """
        self.auto_generate = auto_generate

    def get_auto_generate(self):
        """Set whether auto number generation is enabled. 
    
        Returns:      
            bool: True to enable auto number genration else false.

        """
        return self.auto_generate

    def set_prefix_string(self, prefix_string):
        """Set prefix string.

        Args:
            prefix_string(str): Prefix string.
 
        """
        self.prefix_string = prefix_string

    def get_prefix_string(self):
        """Get prefix string.

        Returns:
            str: Prefix string.
 
        """
        return self.prefix_string

    def set_reference_text(self, reference_text):
        """Set reference text.
 
        Args:
            reference_text(str): Reference text.

        """
        self.reference_text = reference_text

    def get_reference_text(self):
        """Get reference text.

        Returns:
            str: Reference text.

        """
        return self.reference_text

    def set_next_number(self, next_number):
        """Set reference number.

        Args:
            reference_number(str): Reference number.

        """
        self.next_number = next_number

    def get_next_number(self):
        """Get next number.

        Returns:
            str: Next number.

        """
        return self.next_number

    def set_notes(self, notes):
        """Set notes.

        Args:
            notes(str): Notes.

        """
        self.notes = notes

    def get_notes(self):
        """Get notes.

        Returns:
            str: Notes.

        """
        return self.notes

    def set_terms(self, terms):
        """Set terms.

        Args:
            terms(str): Terms.

        """
        self.terms = terms

    def get_terms(self):
        """Get terms.

        Returns:
            str: Terms.

        """
        return self.terms

    def to_json(self):
        """This method is used to convert creditnote setting object to json objcet.
 
        Returns:
            dict: Dictionary containing json object for credit note setting.

        """
        data = {}
        if self.auto_generate is not None:
            data['auto_generate'] = self.auto_generate
        if self.prefix_string != '':
            data['prefix_string'] = self.prefix_string 
        if self.reference_text != '':
            data['reference_text'] = self.reference_text
        if self.next_number != '':
            data['next_number'] = self.next_number
        if self.notes != '':
            data['notes'] = self.notes
        if self.terms != '':
            data['terms'] = self.terms
        return data



#$Id$

class PlaceHolder:
    """This class is used to create object for place holders."""
    def __init__(self):
        """Initialize parameters for place holders."""
        self.invoice = []
        self.customer = []
        self.organization = []

    def set_invoice(self, invoice):
        """Set invoice.

        Args:
            invoice(instance): Invoice object.

        """
        self.invoice.append(invoice)

    def get_invoice(self):
        """Get invoice.

        Returns:
            List of instance: List of Invoice object.

        """
        return self.invoice

    def set_customer(self, customer):
        """Set customer.

        Args:
            customer(instance): Customer object.

        """
        self.customer.append(customer)

    def get_customer(self):
        """Get customer.

        Returns:        
            List of instance: List of customer object.

        """
        return self.customer

    def set_organization(self, organization):
        """Set organization.

        Args:
            organization(instance): Organization object.

        """
        self.organization.append(organization)

    def get_organization(self):
        """Get organization.
       
        Returns:
            List of instance: List of organization object.

        """
        return self.organization




#$Id$#

from books.model.BankAccount import BankAccount
from books.model.BankAccountList import BankAccountList
from books.model.Transaction import Transaction
from books.model.Statement import Statement
from books.model.PageContext import PageContext

class BankAccountsParser:
    """This class is used to parse the json response for Bank accounts."""
  
    def get_list(self, resp):
        """This method parses the given response and returns bank account list 
            object.

        Args:
            resp(dict): Response containing json object for Bank account list.

        Returns:
            instance: Bank account list object.

        """
        bank_account_list = BankAccountList()
        for value in resp['bankaccounts']:
            bank_accounts = BankAccount()
            bank_accounts.set_account_id(value['account_id'])
            bank_accounts.set_account_name(value['account_name'])
            bank_accounts.set_currency_id(value['currency_id'])
            bank_accounts.set_currency_code(value['currency_code'])
            bank_accounts.set_account_type(value['account_type'])
            bank_accounts.set_uncategorized_transactions(\
            value['uncategorized_transactions'])
            bank_accounts.set_is_active(value['is_active'])
            bank_accounts.set_balance(value['balance'])
            bank_accounts.set_bank_name(value['bank_name'])
            bank_accounts.set_routing_number(value['routing_number'])
            bank_accounts.set_is_primary_account(value['is_primary_account'])
            bank_accounts.set_is_paypal_account(value['is_paypal_account'])
            if value['is_paypal_account']:
                bank_accounts.set_paypal_email_address(\
                value['paypal_email_address'])
            bank_account_list.set_bank_accounts(bank_accounts)
        page_context_obj = PageContext()
        page_context = resp['page_context']
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        page_context_obj.set_applied_filter(page_context['applied_filter'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        bank_account_list.set_page_context(page_context_obj)
        return bank_account_list

    def get_account_details(self, resp):
        """This method parses the given response and returns bank account 
            object. 

        Args:
            resp(dict): Response containing json object for bank accounts.

        Returns:
            instance: Bank accounts object.

        Raises: 
            Books Exception: If status is not '200' or '201'.

        """
        bank_account = resp['bankaccount']
        bank_account_obj = BankAccount()
        bank_account_obj.set_account_id(bank_account['account_id'])
        bank_account_obj.set_account_name(bank_account['account_name'])
        bank_account_obj.set_currency_id(bank_account['currency_id'])
        bank_account_obj.set_currency_code(bank_account['currency_code'])
        bank_account_obj.set_account_type(bank_account['account_type'])
        bank_account_obj.set_account_number(bank_account['account_number'])
        bank_account_obj.set_uncategorized_transactions(\
        bank_account['uncategorized_transactions'])
        bank_account_obj.set_is_active(bank_account['is_active'])
        bank_account_obj.set_balance(bank_account['balance'])
        bank_account_obj.set_bank_name(bank_account['bank_name'])
        bank_account_obj.set_routing_number(bank_account['routing_number'])
        bank_account_obj.set_is_primary_account(bank_account[\
        'is_primary_account'])
        bank_account_obj.set_is_paypal_account(bank_account[\
        'is_paypal_account'])
        if bank_account['is_paypal_account']:
            bank_account_obj.set_paypal_email_address(\
            bank_account['paypal_email_address'])
        bank_account_obj.set_description(bank_account['description'])
        return bank_account_obj

    def get_message(self, resp):
        """This message parses the given response and returns message string.
    
        Args:
            resp(dict): Response containing json object for message.

        Returns:
            str: Success message.

        """
        return resp['message']

    def get_statement(self, resp):
        """This method parses the given response and returns statement object.

        Args:
            resp(dict): Response containing json onbject for statement.

        Returns: 
            instance: Statement object.

        """
        statement = resp['statement']
        statement_obj = Statement()
        statement_obj.set_statement_id(statement['statement_id'])
        statement_obj.set_from_date(statement['from_date'])
        statement_obj.set_to_date(statement['to_date'])
        statement_obj.set_source(statement['source'])
        for value in statement['transactions']:
            transactions = Transaction()
            transactions.set_transaction_id(value['transaction_id'])
            transactions.set_debit_or_credit(value['debit_or_credit'])
            transactions.set_date(value['date'])
            transactions.set_customer_id(value['customer_id'])
            transactions.set_payee(value['payee'])
            transactions.set_reference_number(value['reference_number'])
            transactions.set_amount(value['amount'])
            transactions.set_status(value['status'])
            statement_obj.set_transactions(transactions)
        page_context = resp['page_context']
        page_context_obj = PageContext()
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        statement_obj.set_page_context(page_context_obj)
        return statement_obj
        
            



#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.ContactParser import ContactParser
from .Api import Api
from json import dumps

base_url = Api().base_url + 'contacts/'
parser = ContactParser()
zoho_http_client = zoho_http_client = ZohoHttpClient()

class ContactPersonsApi:
    """ContactPersonsApi class is used to:

    1.To get the list of contact persons of a contact with pagination.
    2.To get the details of a contact person.
    3.To create a contact person for a contact.
    4.To update an existing contact person.
    5.To delete a contact person.
    6.To mark a contact person as primary for the contact.

    """
 
    def __init__(self, authtoken, organization_id):
        """Initialize ContactPersons Api using user's authtoken and 
            organization id.

        Args:
            authtoken(str): User's authtoken.
            organization_id(str): User's organization id.

        """
        self.details={
            'authtoken':authtoken,
            'organization_id':organization_id,
            }

    def get_contact_persons(self, contact_id):
        """List contact persons of a contact with pagination.

        Args:
            contact_id(str): Contact id of a contact.

        Returns:
            instance: Contact Persons list object.

        """ 
        url = base_url + contact_id + '/contactpersons'
        response=zoho_http_client.get(url, self.details)
        return parser.get_contact_persons(response) 

    def get(self, contact_person_id):
        """Get the contact persons details.

        Args:
            contact_id(str): Contact id of a contact.

        Returns:
            instance: Contact Person object.

        """
        url = base_url + 'contactpersons/' + contact_person_id
        response = zoho_http_client.get(url, self.details)
        return parser.get_contact_person(response) 
  
    def create(self, contact_person):
        """Create a contact person for contact.

        Args:
            contact_id(str): Contact_id of a contact.

        Returns:
            instance: Contact person object.

        """
        url = base_url + 'contactpersons'
        json_object = dumps(contact_person.to_json())
        print(json_object)
        data = {
            'JSONString': json_object
            }
        response = zoho_http_client.post(url, self.details, data)
        return parser.get_contact_person(response)
   
    def update(self, contact_person_id, contact_person):
        """Update an existing contact person.

        Args:
            contact_id(str): Contact_id of a contact.
            contact_person(instance): Contact person object.

        Returns:
            instance: Contact person object.

        """
        url = base_url + 'contactpersons/' + contact_person_id
        json_object = dumps(contact_person.to_json())
        data = {
            'JSONString': json_object
            }
        response = zoho_http_client.put(url, self.details, data)
        return parser.get_contact_person(response) 

    def delete(self, contact_person_id):
        """Delete a contact person.
        
        Args:
            contact_person_id(str): Contact person id of a contact person.
    
        Returns:
            str: Success message('The contact person has been deleted').
 
        """
        url = base_url + 'contactpersons/' + contact_person_id
        response = zoho_http_client.delete(url, self.details)
        return parser.get_message(response)
      
    def mark_as_primary_contact(self, contact_person_id):
        """Mark a contact person as primary for the contact.
        
        Args:
            contact_person_id(str): Contact person id of a contact person.
       
        Returns:
            str: Success message('This contact person has been marked as 
                your primary contact person.').
      
        """
        url = base_url + 'contactpersons/' + contact_person_id + '/primary'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response) 
    
  
   



#$Id$#

class Api:
     """This class is used to maintain the service URL."""
     
     def __init__(self):
         """Initialize parameters for API class."""
 
         self.base_url = 'https://books.zoho.in/api/v3/'

    
     



#$Id$

class FromEmail:
    """This class is used to create object for FromEmails."""
    def __init__(self):
        """Initialize parameters for FromEmails."""
        self.user_name = '' 
        self.selected = None
        self.email = '' 
        self.is_org_email_id = None

    def set_user_name(self, user_name):
        """Set the user name.
         
        Args:
            user_name(str): User name. 
        """
        self.user_name = user_name

    def get_user_name(self):
        """Get the user name.
       
        Returns:
            str: User name.
      
        """ 
        return self.user_name
   
    def set_selected(self, selected):
        """Set selected.
        
        Args:
            selected(bool): True if selected else False.
 
        """
        self.selected = selected
 
    def get_selected(self):
        """Get selected.
       
        Returns:
            bool: True if selected else False.
   
        """
        return self.selected
  
    def set_email(self, email):
        """Set email address.
       
        Args:
            email(str): Email address.
       
        """
        self.email = email

    def get_email(self):
        """Get email address.
       
        Returns:
            str: Email address.
 
        """
        return self.email
  
    def set_is_org_email_id(self, is_org_email_id):
        """Set whether it is organization's email id or not.
         
        Args:
            is_org_email_id(bool): True if it is organization's mail 
                id else False.

        """
        self.is_org_email_id = is_org_email_id

    def get_is_org_email_id(self):
        """Get whether it is organization's mail id or not.
 
        Returns:
            bool: True if it is organization's mail id else False.

        """
        return self.is_org_email_id



#$Id$

class Invoice:
    """This class is used to create object for invoice."""
    def __init__(self):
        """Initialize parameters for Invoice object."""
        self.invoice_id = ''
        self.invoice_number = ''
        self.date = ''
        self.status = ''
        self.payment_terms = 0
        self.payment_terms_label = ''
        self.due_date = ''
        self.payment_expected_date = ''
        self.last_payment_date = ''
        self.reference_number = ''
        self.customer_id = ''
        self.customer_name = ''
        self.contact_persons = []
        self.currency_id = ''
        self.currency_code = ''
        self.exchange_rate = 0.00
        self.discount = ''
        self.is_discount_before_tax = None
        self.discount_type = ''
        self.recurring_invoice_id = ''
        self.line_items = []
        self.shipping_charge = 0.00
        self.adjustment = 0.0
        self.adjustment_description = ''
        self.sub_total = 0.0
        self.tax_total = 0.0
        self.total = 0.0
        self.taxes = []
        self.payment_reminder_enabled = None
        self.payment_made = 0.0
        self.credits_applied = 0.0
        self.tax_amount_withheld = 0.0
        self.balance = 0.0
        self.write_off_amount = 0.0
        self.allow_partial_payments = None
        self.price_precision = 2
        self.payment_options = {
            'payment_gateways': []
            }
        self.is_emailed = False
        self.reminders_sent = 0
        self.last_reminder_sent_date = ''
        self.billing_address = ''
        self.shipping_address = ''
        self.notes = ''
        self.terms = ''
        self.custom_fields = []
        self.template_id = ''
        self.template_name = ''
        self.created_time = ''
        self.last_modified_time = ''
        self.attachment_name = ''
        self.can_send_in_mail = None
        self.salesperson_id = ''
        self.salesperson_name = ''
        self.invoice_url = ''
        self.invoice_payment_id = ''
        self.due_days = ''
        self.custom_body = ''
        self.custom_subject = ''
        self.invoiced_estimate_id = ''
        self.amount_applied = ''
        self.name = ''
        self.value = ''
        self.gst_no = ''
        self.reason = ''

    
    def set_name(self, name):
        """Set name.

        Args:
            name(str): Name.

        """
        self.name = name

    def get_name(self):
        """Get name.

        Returns:
            str: Name.

        """
        return self.name

    def set_value(self,value):
        """Set value.

        Args:
            value(str): Value.

        """
        self.value = value

    def get_value(self):
        """Get value.

        Returns:
            str: Value.

        """
        return self.value
  
    def set_invoice_id(self, invoice_id):
        """Set invoice id.

        Args: 
            invoice_id(str): Invoice id.

        """
        self.invoice_id = invoice_id

    def get_invoice_id(self):
        """Get invoice id.

        Returns:
            str: Invoice id.

        """
        return self.invoice_id
 
    def set_invoice_number(self, invoice_number):
        """Set invoice number.

        Args:
            invoice_numeber(str): Invoice number.
 
        """
        self.invoice_number = invoice_number
 
    def get_invoice_number(self):
        """Get invoice number.
        
        Returns:
            str: Invoice number.
  
        """
        return self.invoice_number

    def set_date(self, date):
        """Set date at which the invoice is created.
 
        Args:
            date(str): Date at which invoice is created.

        """
        self.date = date

    def get_date(self):
        """Get date at which invoice is created.

        Returns:
            str:  Date at which invoice is created.

        """
        return self.date
   
    def set_status(self, status): 
        """Set status.

        Args:
            status(str): Status.

        """
        self.status = status
 
    def get_status(self):
        """Get status.

        Returns:
            str: Status.

        """
        return self.status

    def set_payment_terms(self, payment_terms):
        """Set payment terms in days.Invoice due date will be calculated based on this.
      
        Args:
            payment_terms(int): Payment terms in days.Eg 15, 30, 60.

        """ 
        self.payment_terms = payment_terms

    def get_payment_terms(self):
        """Get payment terms.

        Returns:
            int: Payment terms in days.

        """
        return self.payment_terms

    def set_payment_terms_label(self, payment_terms_label):
        """Set Payments terms label. 

        Args:
            payment_terms_label(str): Payment terms label. Default value for 15 days is  "Net 15".

        """         
        self.payment_terms_label = payment_terms_label

    def get_payment_terms_label(self):
        """Get payment terms label.

        Returns:
            str: PAyment terms label.

        """
        return self.payment_terms_label
  
    def set_due_date(self, due_date):
        """Set due date for invoice.

        Args:
            due_date(str): Due date for invoice. Format is yyyy-mm-dd.

        """
        self.due_date = due_date

    def get_due_date(self):
        """Get due date of invoice.

        Returns:
            str: Due date dor invoice.

        """
        return self.due_date
   
    def set_payment_expected_date(self, payment_expected_date):
        """Set payment expected date.

        Args:
            payment_expected_date(str): Payment expected date.

        """
        self.payment_expected_date = payment_expected_date

    def get_payment_expected_date(self): 
        """Get payment expected date.

        Returns:
            str: Payment expected date.

        """
        return self.payment_expected_date

    def set_last_payment_date(self, last_payment_date):
        """Set last payment date.
 
        Args:
            last_payment_date(str): last_payment_date

        """
        self.last_payment_date = last_payment_date

    def get_last_payment_date(self):
        """Get last payment date.
 
        Returns:
            str: last payment date.

        """
        return self.last_payment_date
   
    def set_reference_number(self, reference_number):
        """Set reference number for the customer payment.
 
        Args:
            reference_number(str): Reference number.

        """
        self.reference_number = reference_number
 
    def get_reference_number(self): 
        """Get reference number for the customer payment.

        Returns:
            str: Reference number.

        """
        return self.reference_number

    def set_customer_id(self, customer_id):
        """Set customer id.

        Args:
            customer_id(str): Customer id.
 
        """
        self.customer_id = customer_id

    def get_customer_id(self):
        """Get customer id.

        Returns:
            str: Customer id.

        """
        return self.customer_id

    def set_customer_name(self, customer_name):
        """Set customer name.

        Args:
            customer_name(str): Customer name.
         
        """
        self.customer_name = customer_name

    def get_customer_name(self):
        """Get customer name.

        Returns:
            str: Customer name.

        """
        return self.customer_name

    def set_contact_persons(self, contact_person):
        """Set the list of ID of contact persons for which thank you mail has to be sent.

        Args:
            contact_peson(list): Id of contact persons for which thank you mail has to be sent.

        """
        self.contact_persons.extend(contact_person)

    def get_contact_persons(self):
        """Get the Id of contact persons for which thank you mail has to be sent.

        Returns: 
            list: List of ID of contact persons.

        """
        return self.contact_persons

    def set_currency_id(self, currency_id):
        """Set currency id.

        Args: 
            currency_id(str): Currency id.

        """
        self.currency_id = currency_id

    def get_currency_id(self):
        """Get currency id.

        Returns:
            str: Currency id.

        """
        return self.currency_id

    def set_currency_code(self, currency_code):
        """Set currency code.
  
        Args:
            currency_code(str): Currency code.

        """
        self.currency_code = currency_code 

    def get_currency_code(self):
        """Get currency code.

        Returns:
            str: Currency code.

        """
        return self.currency_code
  
    def set_exchange_rate(self, exchange_rate): 
        """Set exchange rate for the currency.
  
        Args:
            exchange_rate(int): Exchange rate.
        
        """  
        self.exchange_rate = exchange_rate

    def get_exchange_rate(self):
        """Get exchange rate of the currency.
        
        Returns:
            int: Exchange rate.

        """
        return self.exchange_rate

    def set_discount(self, discount):
        """Set discount applied to the invoice.
         
        Args: 
            discount(str): Discount. It can be either in % or in amount.
   
        """
        self.discount = discount

    def get_discount(self):
        """Get discount applied to the invoice.
 
        Returns:
            str: Discount.
        
        """
        return self.discount

    def set_is_discount_before_tax(self, is_discount_before_tax):
        """Set whether to discount before tax or after tax.

        Args:
            is_discount_before_tax(bool): True to discount before tax else False to discount after tax.

        """
        self.is_discount_before_tax = is_discount_before_tax
     
    def get_is_discount_before_tax(self):
        """Get whether to discount before tax or after tax.
 
        Returns:
            bool: True to discount before tax else False to discount after tax.

        """
        return self.is_discount_before_tax
   
    def set_discount_type(self, discount_type):
        """Set how the discount should be specified.
 
        Args:
            discount_type: Discount type. Allowed values are entity_level or item_level

        """
        self.discount_type = discount_type

    def get_discount_type(self):
        """Get discount type.
        
        Returns:
            str: Discount type.
 
        """
        return self.discount_type
   
    def set_recurring_invoice_id(self, recurring_invoice_id):
        """Set id of the recurring invoice from which the invoice is created.
 
        Args:
            recurring_invoice_id(str): Recurring invoice id.
 
        """
        self.recurring_invoice_id = recurring_invoice_id

    def get_recurring_invoice_id(self):
        """Get id of the recurring invoice from which the invoice is created.

        Returns:
            str: Recurring invoice id.
 
        """
        return self.recurring_invoice_id

    def set_line_items(self, line_item):
        """Set line items for an invoice.
        
        Args: 
            instance: Line item object.

        """
        self.line_items.append(line_item)

    def get_line_items(self):
        """Get line items of an invoice.
        
        Returns:
            list: List of line items object

        """
        return self.line_items
   
    def set_shipping_charge(self, shipping_charge):
        """Set shipping charge for the invoice.
 
        Args:
            shipping_charge(float): Shipping charge.

        """
        self.shipping_charge = shipping_charge
 
    def get_shipping_charge(self):
        """Get shipping charge for the invoice.

        Returns: 
            float: Shipping charge.

        """
        return self.shipping_charge
   
    def set_adjustment(self, adjustment):
        """Set adjustments made to the invoice.
     
        Args:
            adjustment(float): Adjustments made.

        """
        self.adjustment = adjustment

    def get_adjustment(self):
        """Get adjustments made to the invoice.
        
        Returns:
            float: Adjustments made.

        """
        return self.adjustment
   
    def set_adjustment_description(self, adjustment_description):
        """Set to customize adjustment description.
        
        Args: 
            adjustment_description(str): Adjustment description.

        """
        self.adjustment_description = adjustment_description

    def get_adjustment_description(self):
        """Get adjustment description.

        Returns:
            str: Adjustment description.
      
        """
        return self.adjustment_description

    def set_sub_total(self, sub_total):
        """Set sub total.

        Args:
            sub_total(float): Sub total.
 
        """
        self.sub_total = sub_total

    def get_sub_total(self):
        """Get sub total.

        Returns:
            float: Sub total.

        """
        return self.sub_total

    def set_tax_total(self, tax_total):
        """Set tax total.
        
        Args: 
            tax_total(float): Tax total.
  
        """
        self.tax_total = tax_total

    def get_tax_total(self):
        """Get tax total.

        Returns:
            float: Tax total.

        """
        return self.tax_total
  
    def set_total(self, total):
        """Set total.
    
        Args:
            total(float): Total amount.
 
        """
        self.total = total

    def get_total(self):
        """Get total.
      
        Returns:
            float: Total amount.
 
        """
        return self.total
  
    def set_taxes(self, tax):
        """Set taxes.

        Args:
            tax(instance): Tax.

        """ 
        self.taxes.append(tax)
 
    def get_taxes(self):
        """Get taxes.
        
        Returns:
            list: List of tax objects.

        """
        return self.taxes

    def set_payment_reminder_enabled(self, payment_reminder_enabled):
        """Set whether to enable payment reminder or not.

        Args:
            payment_reminder_enabled(bool): True to enable payment reminder else False.

        """
        self.payment_reminder_enabled = payment_reminder_enabled

    def get_payment_reminder_enabled(self):
        """Get whether payment reminder is enabled or not.

        Returns:
            bool: True if payment reminder is enabled else False.

        """
        return self.payment_reminder_enabled

    def set_payment_made(self, payment_made):
        """Set payment made.

        Args:
            payment_made(float): Payments made.

        """
        self.payment_made = payment_made

    def get_payment_made(self):
        """Get payments made.

        Returns:
            float: Payments made.

        """
        return self.payment_made
 
    def set_credits_applied(self, credits_applied):
        """Set credits applied.

        Args: 
            credits_applied(float): Credits applied.
 
        """
        self.credits_applied = credits_applied

    def get_credits_applied(self): 
        """Get credits applied.
    
        Returns:
            float: Credits applied.
      
        """
        return self.credits_applied

    def set_tax_amount_withheld(self, tax_amount_withheld): 
        """Set tax amount withheld.

        Args:
            tax_amount_withheld(float): Tax amount withheld.

        """
        self.tax_amount_withheld = tax_amount_withheld

    def get_tax_amount_withheld(self):
        """Get tax amount withheld.

        Returns:
            float: Tax amount withheld.

        """
        return self.tax_amount_withheld

    def set_balance(self, balance):
        """Set balance.

        Args:
            balance(float): Balance.

        """
        self.balance = balance

    def get_balance(self):
        """Get balance.

        Returns:
            float: Balance.

        """
        return self.balance

    def set_write_off_amount(self, write_off_amount):
        """Set write off amount.
     
        Args:
            write_off_amount(float): Write off amount.

        """ 
        self.write_off_amount = write_off_amount
 
    def get_write_off_amount(self):
        """Get write off amount.
 
        Returns:
            float: Write off amount.
      
        """
        return self.write_off_amount

    def set_allow_partial_payments(self, allow_partial_payments):
        """Set whether to allow partial payments or not.

        Args:
            allow_partial_payments(bool): True to allow partial payments else False.
  
        """
        self.allow_partial_payments = allow_partial_payments

    def get_allow_partial_payments(self):
        """Get whether to allow partial payments.
  
        Returns:
            bool: True if partial payments are allowed else False.

        """
        return self.allow_partial_payments

    def set_price_precision(self, price_precision):
        """Set price precision.

        Args: 
            price_precision(int): Price precision.
        
        """
        self.price_precision = price_precision 
 
    def get_price_precision(self):
        """Get price precision.
       
        Returns:
            int: Price precision.
 
        """
        return self.price_precision

    def set_payment_options(self, payment_gateways):
        """Set payment options.

        Args: 
            payment_gateways(instance): Online payment gateways through which payment can be made.
        
        """
        self.payment_options['payment_gateways'].append(payment_gateways)

    def get_payment_options(self):
        """Get payment options.
 
        Returns:
            dict: Payment options for the invoice.

        """
        return self.payment_options
  
    def set_is_emailed(self, is_emailed):
        """Set whether to email or not.
        
        Args: 
            is_emailed(bool): True to email else False.

        """
        self.is_emailed = is_emailed

    def get_is_emailed(self):
        """Get whether to email.
        
        Returns:
            bool: True to email else False.
 
        """
        return self.is_emailed
   
    def set_reminders_sent(self, reminders_sent):
        """Set reminders sent. 
      
        Args:
            reminders_sent(int): Reminders sent.

        """
        self.reminders_sent = reminders_sent

    def get_reminders_sent(self):
        """Get reminders sent.

        Returns:
            int: Reminders sent.

        """
        return self.reminders_sent

    def set_last_reminder_sent_date(self, last_reminder_sent_date):
        """Set last reminder sent date.
     
        Args:
            last_reminder_sent_date(str): Last reminder sent date

        """
        self.last_reminder_sent_date = last_reminder_sent_date

    def get_last_reminder_sent_date(self):
        """Get last reminder sent date.

        Returns:
            str: Last reminder sent date.
 
        """
        return self.last_reminder_sent_date

    def set_billing_address(self, billing_address):
        """Set billing address.

        Args:
            billing_address(instance): Billing address object.

        """
        self.billing_address = billing_address
 
    def get_billing_address(self):
        """Get billing address.
        
        Returns:
            instance: Billing address object.
      
        """
        return self.billing_address
    
    def set_shipping_address(self, shipping_address):
        """Set shipping address.
 
        Args:
            shipping_address(instance): Shipping address object.
 
        """
        self.shipping_address = shipping_address
 
    def get_shipping_address(self):
        """Get shipping address.

        Returns:
            instance: Shipping address object.

        """
        return self.shipping_address

    def set_notes(self, notes):
        """Set notes.

        Args: 
            notes(str): Notes.

        """
        self.notes = notes

    def get_notes(self):
        """Get notes.

        Returns: 
            str: Notes

        """
        return self.notes

    def set_terms(self, terms):
        """Set terms.

        Args: 
            terms(str): Terms.
  
        """
        self.terms = terms

    def get_terms(self):
        """Get terms.

        Returns:
            str: Terms.

        """
        return self.terms

    def set_custom_fields(self, custom_field):
        """Set custom fields.

        Args:
            custom_field(instance): custom field object.
 
        """
        self.custom_fields.append(custom_field)
 
    def get_custom_fields(self):
        """Get custom field.

        Returns: 
            instance: custom field object.
 
        """
        return self.custom_fields
    
    def set_template_id(self, template_id):
        """Set template id.

        Args:
            template_id(str): Template id.

        """
        self.template_id = template_id

    def get_template_id(self):
        """Get template id.

        Returns:
            str: Template id.

        """
        return self.template_id

    def set_template_name(self, template_name):
        """Set template name.

        Args: 
            template_name(str): Template name.

        """
        self.template_name = template_name

    def get_template_name(self):
        """Get template name.
        
        Returns: 
            str: Template name.

        """
        return self.template_name

    def set_created_time(self, created_time):
        """Set created time.

        Args: 
            created_time(str): Created time.

        """
        self.created_time = created_time
 
    def get_created_time(self):
        """Get created time.

        Returns:
            str: Created time.

        """
        return self.created_time

    def set_last_modified_time(self, last_modified_time):
        """Set last modified time.
 
        Args:
            last_modified_time(str): Last modified time.

        """
        self.last_modified_time = last_modified_time

    def get_last_modified_time(self):
        """Get last modified time.

        Returns:
            str: Last modified time.

        """
        return self.last_modified_time
  
    def set_attachment_name(self, attachment_name):
        """Set attachment name.

        Args:
            attachment_name(str): Attachment name.
     
        """
        self.attachment_name = attachment_name
 
    def get_attachment_name(self):
        """Get attachment name.

        Returns:
            str: Attachment name.

        """
        return self.attachment_name

    def set_can_send_in_mail(self, can_send_in_mail):
        """Set whether the invoice can be sent in mail or not.

        Args:
            can_send_in_mail(bool): True if it can be sent in mail else False.

        """
        self.can_send_in_mail = can_send_in_mail

    def get_can_send_in_mail(self):
        """Get whether the invoice can be sent in mail or not.

        Returns:
            bool: True if it can be sent in mail else False.

        """
        return self.can_send_in_mail
  
    def set_salesperson_id(self, salesperson_id):
        """Set salesperson id.

        Args:
            salesperson_id(str): Salesperson id.

        """
        self.salesperson_id = salesperson_id
 
    def get_salesperson_id(self):
        """Get salesperson id.

        Returns:
            str: Salesperson id.
        
        """
        return self.salesperson_id
    
    def set_salesperson_name(self, salesperson_name):
        """Set salesperson name.

        Args: 
            salesperson_name(str): Salesperson name.
  
        """
        self.salesperson_name = salesperson_name
 
    def get_salesperson_name(self):
        """Get salesperson name.

        Returns:
            str: Salesperson name.

        """
        return self.salesperson_name

    def set_invoice_url(self, invoice_url):
        """Set invoice url.

        Args:
            invoice_url(str): Invoice url.
 
        """
        self.invoice_url = invoice_url
 
    def get_invoice_url(self):
        """Get invoice url.
       
        Returns:
            str: Invoice url.

        """
        return self.invoice_url
  
    def set_due_days(self, due_days):
        """Set due days.

        Args:
            due_days(str): Due days.

        """
        self.due_days = due_days
 
    def get_due_days(self):
        """Get due days.

        Returns:
            str: Due days.

        """
        return self.due_days

    def set_custom_body(self, custom_body):
        """Set custom body.

        Args:
            custom_body(str): Custom body.

        """
        self.custom_body = custom_body
 
    def get_custom_body(self):
        """Get custom body.

        Returns:
            str: Custom body.

        """
        return self.custom_body

    def set_custom_subject(self, custom_subject):
        """Set custom subject.

        Args:
            custom_subject(str): Custom subject.

        """
        self.custom_subjecct = custom_subject
 
    def get_custom_subject(self):
        """Get custom subject.

        Returns:
            str: Custom subject.

        """
        return self.custom_subject

    def set_invoiced_estimate_id(self, invoiced_estimate_id):
        """Set invoiced estimate id.
 
        Args:
            invoiced_estimate_id(str): Invoiced estimate id.

        """
        self.invoiced_estimate_id = invoiced_estimate_id

    def get_invoiced_estimate_id(self): 
        """Get invoiced estimate id.

        Returns:
            str: Invoiced estimate id.

        """
        return self.invoiced_estimate_id
   
    def set_amount_applied(self, amount_applied):
        """Set amount applied.
  
        Args: 
            amount_applied(str): Amount applied.
 
        """
        self.amount_applied = amount_applied

    def get_amount_applied(self):
        """Get amount applied.

        Returns:
            str: Amount applied.

        """
        return self.amount_applied
 
    def set_invoice_payment_id(self, invoice_payment_id):
        """Set invoice payment id.

        Args:
            invoice_payment_id(str): Invoice payment id.

        """
        self.invoice_payment_id = invoice_payment_id

    def get_invoice_payment_id(self):
        """Get invoice payment id.

        Returns:
            str: Invoice payment id.

        """
        return self.invoice_payment_id

    def to_json(self):
        """This method is used to convert invoice object to json object.

        Returns:
            dict: Dictionary containing json object for invoices.

        """
        data = {}
        if self.customer_id != '':
            data['customer_id'] = self.customer_id
        if self.contact_persons:
            data['contact_persons'] = self.contact_persons
        if self.reference_number != '':
            data['reference_number'] = self.reference_number
        if self.template_id != '':
            data['template_id'] = self.template_id
        if self.date != '':
            data['date'] = self.date
        if self.payment_terms > 0:
            data['payment_terms'] = self.payment_terms
        if self.payment_terms_label != '':
            data['payment_terms_label'] = self.payment_terms_label
        if self.due_date != '':
            data['due_date'] = self.due_date
        if self.discount != '':
            data['discount'] = self.discount
        if self.is_discount_before_tax is not None:
            data['is_discount_before_tax'] = self.is_discount_before_tax 
        if self.discount_type != '':
            data['discount_type'] = self.discount_type
        if self.exchange_rate > 0:
            data['exchange_rate'] = self.exchange_rate
        if self.recurring_invoice_id != '':
            data['recurring_invoice_id'] = self.recurring_invoice_id
        if self.invoiced_estimate_id != '':
            data['invoiced_estimate_id'] = self.invoiced_estimate_id
        if self.salesperson_name != '':
            data['salesperson_name'] = self.salesperson_name
        if self.custom_fields:
            data['custom_fields'] = []
            for value in self.custom_fields: 
                custom_field = value.to_json()
                data['custom_fields'].append(custom_field)
        if self.line_items:
            data['line_items'] = []
            for value in self.line_items:
                line_item = value.to_json()
                data['line_items'].append(line_item)
        if self.payment_options['payment_gateways']:
            data['payment_options'] = {'payment_gateways':[]}
            for value in self.payment_options['payment_gateways']:
                payment_gateway = value.to_json()
                data['payment_options']['payment_gateways'].append(payment_gateway)
        if self.allow_partial_payments != None:
            data['allow_partial_payments'] = self.allow_partial_payments
        if self.custom_body != '':
            data['custom_body'] = self.custom_body
        if self.custom_subject != '':
            data['custom_subject'] = self.custom_subject
        if self.notes != '':
            data['notes'] = self.notes
        if self.terms != '':
            data['terms'] = self.terms
        if self.shipping_charge > 0:
            data['shipping_charge'] = self.shipping_charge
        if self.adjustment > 0: 
            data['adjustment'] = self.adjustment
        if self.adjustment_description != '':
            data['adjustment_description'] = self.adjustment_description
        if self.invoice_number != '':
            data['invoice_number'] = self.invoice_number
        if self.reason != '':
            data['reason'] = self.reason
        if self.invoice_id != '':
            data['invoice_id'] = self.invoice_id
        if self.amount_applied != '':
            data['amount_applied'] = self.amount_applied
        if self.total != '':
            data['total'] = self.total
        return data



#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.RecurringInvoiceParser import RecurringInvoiceParser
from .Api import Api
from json import dumps

base_url = Api().base_url + 'recurringinvoices/'
parser = RecurringInvoiceParser()
zoho_http_client = ZohoHttpClient()

class RecurringInvoicesApi:
    """Recurring invoice api class is used:
    
    1.To list all the recurring invoices with pagination.
    2.To get details of a recurring invoice.
    3.To create a recurring invoice.
    4.To update an existing recurring invoice.
    5.To delete an existing recurring invoice.
    6.To stop an active recurring invoice.
    7.To resume a stopped recurring invoice.
    8.To update the pdf template associated with the recurring invoice.
    9.To get the complete history and comments of a recurring invoice.

    """

    def __init__(self, authtoken, organization_id):
        """Initialize Contacts Api using user's authtoken and organization id.

        Args:
            authtoken(str): User's authtoken.
            organization_id(str): User's organization id.

        """
        self.details = {
            'authtoken': authtoken, 
            'organization_id': organization_id
            }

    def get_recurring_invoices(self, parameter=None): 
        """List of recurring invoices with pagination.

        Args:
            parameter(dict, optional): Filter with which the list has to be 
                displayed. Defaults to None.

        Returns:
            instance: Recurring invoice list object.

        """
        response = zoho_http_client.get(base_url, self.details, parameter) 
        return parser.recurring_invoices(response)  
  
    def get_recurring_invoice(self, recurring_invoice_id):
        """Get recurring invoice details.
        
        Args:
            recurring_invoice_id(str): Recurring invoice id.

        Returns:
            instance: Recurring invoice object.

        """
        url = base_url + recurring_invoice_id
        response = zoho_http_client.get(url, self.details)
        return parser.recurring_invoice(response)  

    def create(self, recurring_invoice):
        """Create recurring invoice.
        
        Args:
            recurring_invoice(instance): Recurring invoice object.

        Returns:
            instance: Recurring invoice object.

        """
        json_object = dumps(recurring_invoice.to_json())
        data = {
            'JSONString': json_object
            }
        response = zoho_http_client.post(base_url, self.details, data)
        return parser.recurring_invoice(response)  

    def update(self, recurring_invoice_id, recurring_invoice):
        """Update an existing recurring invoice.

        Args:
            recurring_invoice_id(str): Recurring invoice id.
            recurring_invoice(instance): Recurring invoice object.

        Returns:
            instance: Recurring invoice object.

        """
        url = base_url + recurring_invoice_id
        json_object = dumps(recurring_invoice.to_json())
        data = {
            'JSONString': json_object
            }
        response = zoho_http_client.put(url, self.details, data)
        return parser.recurring_invoice(response)  
    
    def delete(self, recurring_invoice_id): 
        """Delete an existing recurring invoice.
     
        Args:
            recurring_invoice_id(str): Recurring invoice id.
 
        Returns:
            str: Success message('The recurring invoice has been deleted.').
 
        """
        url = base_url + recurring_invoice_id
        response = zoho_http_client.delete(url, self.details)
        return parser.get_message(response) 

    def stop_recurring_invoice(self, recurring_invoice_id):
        """Stop an active recurring invoice.
      
        Args: 
            recurring_invoice_id(str): Recurring invoice id.

        Returns:
            str: Success message ('The recurring invoice has been stopped.').

        """
        url = base_url + recurring_invoice_id + '/status/stop'
        response = zoho_http_client.post(url, self.details, '') 
        return parser.get_message(response)  

    def resume_recurring_invoice(self, recurring_invoice_id):
        """Resume an active recurring invoice.
      
        Args: 
            recurring_invoice_id(str): Recurring invoice id.

        Returns:
            str: Success message ('The recurring invoice has been activated.').

        """
        url = base_url + recurring_invoice_id + '/status/resume'
        response = zoho_http_client.post(url, self.details, '') 
        return parser.get_message(response) 

    def update_recurring_invoice_template(self, 
                                          recurring_invoice_id, template_id):
        """Update the pdf template associated with the recurring invoice.
      
        Args: 
            recurring_invoice_id(str): Recurring invoice id.
            template_id(str): Template id.

        Returns:
            str: Success message ('Recurring invoice information has been
                                  updated.').

        """
        url = base_url + recurring_invoice_id + '/templates/' + template_id
        response = zoho_http_client.put(url, self.details, '')
        return parser.get_message(response)  
 
    def list_recurring_invoice_history(self, recurring_invoice_id):
        """List the complete history and comments of a recurring invoice.

        Args:
            recurring_invoice_id(str): Recurring invoice id.

        Returns:
            instance: Recurring invoice history and comments list object.

        """
        url = base_url + recurring_invoice_id + '/comments'
        response = zoho_http_client.get(url, self.details)
        return parser.recurring_invoice_history_list(response) 

      
      



#$Id$

from books.model.PageContext import PageContext

class CustomerPaymentList:
    """This class is used to create object for customer payment list."""
    def __init__(self):
        """Initialize parameters for customer payments."""
        self.customer_payments = []
        self.page_context = PageContext()

    def set_customer_payments(self, customer_payments):
        """Set customer payments.
   
        Args:
            customer_payments(instance): Customer Payments object.

        """
        self.customer_payments.append(customer_payments)

    def get_customer_payments(self):
        """Get customer payments.
        
        Returns:
            list: List of customer payments object.

        """
        return self.customer_payments

    def set_page_context(self,page_context):
        """Set page context.

        Args: 
            page_context(instance): Page context object.

        """
        self.page_context=page_context

    def get_page_context(self):
        """Get page context.

        Returns:
            instance: page context object.
          
        """
        return self.page_context
 



#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.ProjectsParser import ProjectsParser
from books.api.Api import Api
from json import dumps

base_url = Api().base_url + 'projects/'
zoho_http_client = ZohoHttpClient()
parser = ProjectsParser()

class ProjectsApi:
    """This class is used to 
 
    1. List all projects with pagination.
    2.Get details of a project.
    3.Create a new project.
    4.Update an existing project.
    5.Delete an existing project.
    6.Mark a project as active.
    7.Mark a project as inactive.
    8.Clone a project.
    9.Get list of tasks added to a project.
    10.Get details of a task.
    11.Add task to a project.
    12.Update details of a task.
    13.Delete a task added to a project.
    14.Get list of users associated with a project.
    15.Get details of a user in a project.
    16.Assign users to a project.
    17.Invite and add user to the project.
    18.Update details of a user.
    19.Remove user from the project.
    20.List all time entries with pagination.
    21.Get details of a time entry.
    22.Logging time entries.
    23.Update logged time entry.
    24.Deleting logged time entry.
    25.Deleting time entries.
    26.Get current running timer.
    27.Start tracking time spent.
    28.Stop tracking time.
    29.Get comments for a project.
    30.Post comment to a project.
    31.Delete a comment from a project.
    32.List invoices created for this project.

    """

    def __init__(self, authtoken, organization_id):
        """Initialize Projects api using user's authtoken and organization id.

        Args:
            authtoken(str): User's authtoken.
            organization_id(str): User's organization id.

        """
        self.details = {
            'authtoken': authtoken, 
            'organization_id': organization_id
            }
        
    def get_projects(self, parameters=None): 
        """List all projects with pagination.

        Args:
            parameters(dict, optional): Filter with which the list has to be 
                displayed.

        Returns:
            instance: Projects list object.

        """
        resp = zoho_http_client.get(base_url, self.details, parameters) 
        return parser.get_projects_list(resp)
 
    def get_project(self, project_id):
        """Get details of a project.

        Args:
            project_id(str): Project id.

        Returns:
            instance: Project object.

        """
        url = base_url + project_id
        resp = zoho_http_client.get(url, self.details) 
        return parser.get_project(resp) 

    def create_project(self, project):
        """Create a new project.

        Args:
            project(instance): Project object.

        Returns:
            instance: Projects object

        """
        json_object = dumps(project.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.post(base_url, self.details, data) 
        return parser.get_project(resp) 

    def update_project(self, project_id, project):
        """Update an existing project.

        Args:
            project_id(str): Project id.
            project(instance): Project object.

        Returns: 
            instance: Project object.

        """
        url = base_url + project_id
        json_object = dumps(project.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data) 
        return parser.get_project(resp) 

    def delete_project(self, project_id):
        """Delete an existing project.

        Args:
            project_id(str): Project id.

        Returns:
            str: Success message('The project has been deleted.').

        """
        url = base_url + project_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 

    def activate_project(self, project_id):
        """Mark project as active.

        Args:
            project_id(str): Project id.

        Returns: 
            str: Success message('The selected project have been marked as 
                active.').

        """
        url = base_url + project_id + '/active'
        resp = zoho_http_client.post(url, self.details, '')
        return parser.get_message(resp) 

    def inactivate_project(self, project_id):
        """Mark project as inactive.

        Args:
            project_id(str): Project id.

        Returns:
            str: Success message('The selected project have been marked as 
                inactive.').

        """
        url = base_url + project_id + '/inactive'
        resp = zoho_http_client.post(url, self.details, '')
        return parser.get_message(resp) 

    def clone_project(self, project_id, project):
        """Cloning a project.

        Args:
            project_id(str): Project id.

        Returns:
            instance: Project object.

        """
        url = base_url + project_id +  '/clone'
        json_object = dumps(project.to_json())
        data = {
            'JSONString': json_object
            }
        print(data)
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_project(resp) 

    def get_tasks(self, project_id, sort_column=None):
        """Get list of tasks added to a project.

        Args:
            project_id(str): Project id.
            sort_column(str): Sort column. Allowed Values: task_name, 
                billed_hours, log_time and un_billed_hours.

        Returns: 
            instance: Task list object.

        """  
        url = base_url + project_id + '/tasks'
        if sort_column is not None:
            parameter = {
                'sort_column': sort_column
                }       
        else:
            parameter = None
        resp = zoho_http_client.get(url, self.details, parameter)
        return parser.get_tasks_list(resp) 

    def get_task(self, project_id, task_id):
        """Get details of a task.

        Args: 
            project_id(str): Project id.
            task_id(str): Task id.

        Returns:
            instance: Task object.

        """
        url = base_url + project_id + '/tasks/' + task_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_task(resp) 
     
    def add_task(self, project_id, task):
        """Add task to project.

        Args:
            project_id(str): Project id.
            task(instance): Task object.

        Returns:
            instance: Task object.

        """
        url = base_url + project_id + '/tasks'
        json_object = dumps(task.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_task(resp) 

    def update_task(self, project_id, task_id, task):
        """Update details of a task.

        Args: 
            project_id(str): Project id.
            task_id(str): Task id.

        Returns:
            instance: Task object.

        """
        url = base_url + project_id + '/tasks/' + task_id
        json_object = dumps(task.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_task(resp) 

    def delete_task(self, project_id, task_id):
        """Delete task added to a project.

        Args:
            project_id(str): Project id.
            task_id(str): Task id.

        Returns:
            str: Success message('The task has been deleted.').

        """
        url = base_url + project_id + '/tasks/' + task_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 

    def get_users(self, project_id):
        """Get list of users associated with a project.

        Args:
            project_id(str): Project id.

        Returns:
            instance: Users list object.

        """
        url = base_url + project_id + '/users'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_users(resp) 

    def get_user(self, project_id, user_id):
        """Get details of a user in a project.

        Args:
            project_id(str): Project id.
            user_id(str): User id.

        Returns:
            instance: USer object.

        Raises:
            Books Exception: If status is not '200' or '201'.

        """
        url = base_url + project_id + '/users/' + user_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_user(resp) 

    def assign_users(self, project_id, users):
        """Assign users to a project.
 
        Args:
            project_id(str): Project id.
            users(list of instance): List of users object.

        Returns:
            instance: Users list object.

        """
        url = base_url + project_id + '/users'
        users_obj = {
            'users': [] 
            }
        for value in users:
            user = value.to_json()
            users_obj['users'].append(user)
        json_object = dumps(users_obj)
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_users(resp) 

    def invite_user(self, project_id, user):
        """Invite and add user to the project.

        Args:
            project_id(str): Project id.
            user(instance): User object.

        Returns:
            instance: User object.

        """
        url = base_url + project_id + '/users/invite'
        print(user.to_json())
        json_object = dumps(user.to_json())
        data = {
            'JSONString': json_object
            }
        print(data)
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_user(resp) 
 
    def update_user(self, project_id, user_id, user):
        """Update details of a user.

        Args:
            project_id(str): Project id.
            user_id(str): User id.
            user(instance): User object.

        Returns:
            instance: User object.

        """
        url = base_url + project_id + '/users/' +user_id
        json_object = dumps(user.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_user(resp)  

    def delete_user(self, project_id, user_id):
        """Remove user from the project.

        Args:
            project_id(str): Project id.
            user_id(str): User id.

        Returns:
            str: Success message('The staff has been removed.').

        """
        url = base_url + project_id + '/users/' + user_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 

    def get_time_entries(self, parameters=None):
        """List all time entries with pagination.

        Args:
            parameters(dict, optional): Filter with which the list has to be 
                displayed.

        Returns:
            instance: Time entries list object.

        """
        url = base_url + 'timeentries'
        resp = zoho_http_client.get(url, self.details, parameters) 
        return parser.get_time_entries_list(resp) 

    def get_time_entry(self, time_entry_id):
        """Get details of time entry.

        Args:
            time_entry_id(str): Time entry id.

        Returns:
            instance: Time entry object.

        """
        url = base_url + 'timeentries/' + time_entry_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_time_entry(resp) 

    def log_time_entries(self, time_entry):
        """Logging time entries.

        Args:
            time_entry(instance): Time entry object.

        Returns:
            instance: Time entry.

        """
        url = base_url + 'timeentries'
        json_object = dumps(time_entry.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_time_entry(resp) 

    def update_time_entry(self, time_entry_id, time_entry):
        """Update logged time entry.

        Args:
            time_entry_id(str): Time entry id.
            time_entry(instance): Time entry object.

        Returns:
            instance: Time entry object.

        """
        url = base_url + 'timeentries/' + time_entry_id
        json_object = dumps(time_entry.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_time_entry(resp) 

    def delete_time_entry(self, time_entry_id):
        """Delete time entry.

        Args:
            time_entry_id(str): Time entry id.

        Returns:
            str: Success message('The time entry has been deleted.').

        """
        url = base_url + 'timeentries/' + time_entry_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 

    def delete_time_entries(self, time_entry_ids):
        """Delete time entries.

        Args:
            time_entry_ids(str): Id of the time entries to be deleted.

        Returns:
            str: Success message('The selected timesheet entries have been 
                deleted.').

        """
        url = base_url + 'timeentries'
        param = {
            'time_entry_ids': time_entry_ids
            }
        resp = zoho_http_client.delete(url, self.details, param) 
        return parser.get_message(resp) 

    def get_timer(self):
        """Get current running timer.

        Returns: 
            instance: Time entry object.

        Raises:
            Books Exception: If status is not '200' or '201'.

        """
        url = base_url + 'timeentries/runningtimer/me'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_time_entry(resp) 
 
    def start_timer(self, timer_entry_id):
        """Start tracking time spent.

        Args: 
            timer_entry_id(str): Timer entry id.

        Returns: 
            instance: Time entry object.

        """
        url = base_url + 'timeentries/' + timer_entry_id + '/timer/start'
        data = {
            'JSONString': ''
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_time_entry(resp)

    def stop_timer(self): 
        """Stop tracking time.

        Returns:
            instance: Time entry object.

        """
        url = base_url + 'timeentries/timer/stop'
        data = {
            'JSONString': ''
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_time_entry(resp)

    def get_comments(self, project_id):
        """Get list of comments for a project.

        Args:
            project_id(str): Project id.
         
        Returns:
            instance: Comments list object.

        """
        url = base_url + project_id + '/comments'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_comments(resp) 

    def post_comment(self, project_id, comment):
        """Post comment to a project.

        Args:
            project_id(str): Project id.
            comment(instance): Comment object.

        Returns:
            instance: Comment object.

        """
        url = base_url + project_id + '/comments'
        data = { 
            'description': comment.get_description()
            }
        json_string = {
            'JSONString': dumps(data)
            }
        resp = zoho_http_client.post(url, self.details, json_string)
        return parser.get_comment(resp) 
     
    def delete_comment(self, project_id, comment_id):
        """Delete an existing comment.

        Args:
            project_id(str): Project id.
            comment_id(str): comment id. 

        Returns: 
            str: Success message('The comment has been deleted.').
 
        """
        url = base_url + project_id + '/comments/' + comment_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 

    def get_invoices(self, project_id, sort_column=None):
        """List invoices created for this project.
  
        Args:
            project_id(str): Project id.

        Returns:
            instance: Invoices list object.

        """
        url = base_url + project_id + '/invoices'
        if sort_column is not None:
            param = {
                'sort_column': sort_column
                } 
        else:
            param = None
        resp = zoho_http_client.get(url, self.details, param)
        return parser.get_invoice_list(resp) 
        
            
          
  
           



#$Id$

class LineItem:
    """This class is used to create object for line items."""
    def __init__(self):
        """Initialize the parameters for line items."""
        self.item_id = ''
        self.line_item_id = ''
        self.name = ''
        self.description = ''
        self.item_order = 0
        self.bcy_rate = 0.0
        self.rate = 0.00
        self.quantity = 0
        self.unit = ''
        self.discount = 0.00
        self.tax_id = ''
        self.tax_name = ''
        self.tax_type = ''
        self.tax_percentage = 0.0
        self.item_total = 0.0
        self.expense_id = ''
        self.expense_item_id = ''
        self.expense_receipt_name = ''
        self.time_entry_ids = ''
        self.project_id = ''
        self.account_id = ''
        self.account_name = ''
        self.line_id = ''
        self.debit_or_credit = ''
        self.amount = 0.0
        self.discount_amount = 0.0
        self.hsn_or_sac = ''

    def set_hsn_or_sac(self,hsn_or_sac):
        """Set hsn_or_sac.

        Args:
            hsn_or_sac(str): hsn_or_sac.

        """
        self.hsn_or_sac = hsn_or_sac

    def get_hsn_or_sac(self):
        """Get hsn_or_sac.
 
        Returns:
            str: hsn_or_sac.

        """
        return self.hsn_or_sac
  
    def set_item_id(self,item_id):
        """Set item id.

        Args:
            item_id(str): Item id.

        """
        self.item_id = item_id

    def get_item_id(self):
        """Get item id.

        Returns:
            str: Item id.

        """
        return self.item_id

    def set_line_item_id(self,line_item_id):
        """Set line item id.

        Args:
            line_item_id(str): line_item_id

        """
        self.line_item_id = line_item_id
 
    def get_line_item_id(self):
        """Get line item id.

        Returns:
            str: line item id.

        """
        return self.line_item_id
  
    def set_name(self,name):
        """Set name of the line item.

        Args: 
            name(str): Name of the line item.

        """
        self.name = name

    def get_name(self):
        """Get name of the line item.

        Returns:
            str: Name of the line item.

        """
        return self.name
   
    def set_description(self,description):
        """Set description of the line item.

        Args:
            description(str): Description of the line item.

        """
        self.description = description
 
    def get_description(self):
        """Get description of the line item.

        Returns:
            str: Descritpion of the line item.

        """
        return self.description
  
    def set_item_order(self,item_order):
        """Set item order.

        Args:
            item_order(int): Item order.

        """
        self.item_order = item_order

    def get_item_order(self):
        """Get item order.

        Returns:
            int: Item order.

        """
        return self.item_order
  
    def set_bcy_rate(self,bcy_rate):
        """Set bcy rate.

        Args: 
            bcy_rate(float): Bcy rate.

        """
        self.bc_rate = bcy_rate

    def get_bcy_rate(self):
        """Get bcy rate.

        Returns:
            float: Bcy rate.

        """
        return self.bcy_rate
  
    def set_rate(self,rate):
        """Set rate.

        Args:
            rate(float): Rate.

        """
        self.rate = rate

    def get_rate(self):
        """Get rate.

        Returns:
            float: Rate.

        """
        return self.rate
 
    def set_quantity(self,quantity):
        """Set quantity.

        Args:
            quantity(float): Quantity.
     
        """ 
        self.quantity = quantity

    def get_quantity(self):
        """Get quantity.

        Returns:
            float: Quantity.

        """
        return self.quantity
  
    def set_unit(self,unit):
        """Set unit.

        Args: 
            unit(str): Unit.

        """
        self.unit = unit

    def get_unit(self):
        """Get unit.

        Returns:
            str: Unit.

        """
        return self.unit
  
    def set_discount(self,discount):
        """Set discount.

        Args: 
            discount(str): Discount.

        """
        self.discount = discount

    def get_discount(self):
        """Get discount.

        Returns:
            str:  Discount.

        """
        return self.discount
  
    def set_tax_id(self,tax_id):
        """Set tax id.
 
        Args:
            tax_id(str): Tax id.

        """
        self.tax_id = tax_id

    def get_tax_id(self):
        """Get tax id.

        Returns:
            str: Tax id.

        """
        return self.tax_id
  
    def set_tax_name(self,tax_name):
        """Set tax name.

        Args: 
            tax_name(str): Tax name.

        """
        self.tax_name = tax_name

    def get_tax_name(self):
        """Get tax name.

        Returns: 
            str: Tax name.

        """
        return self.tax_name
  
    def set_tax_type(self,tax_type):
        """Set tax type.

        Args:
            tax_type(str): Tax type.

        """
        self.tax_type = tax_type

    def get_tax_type(self):
        """Get tax type.

        Returns:
            str: Tax type.

        """
        return self.tax_type
  
    def set_tax_percentage(self,tax_percentage):
        """Set tax percentage.

        Args:
            tax_percentage(str): Tax percentage.

        """
        self.tax_percentage = tax_percentage

    def get_tax_percentage(self):
        """Get tax percentage.

        Returns:
            str: Tax percentage.

        """
        return self.tax_percentage

    def set_item_total(self,item_total):
        """Set item total.

        Args: 
            item_total(int): Item total.

        """
        self.item_total = item_total

    def get_item_total(self):
        """Get item total.

        Returns:
            int: Item total.

        """
        return self.item_total

    def set_expense_id(self,expense_id):
        """Set expense id.

        Args:
            expense_id(str): Expense id.

        """
        self.expense_id = expense_id

    def get_expense_id(self):
        """Get expense id.

        Args:
            expense_id(str): Expense id.

        """
        return self.expense_id
   
    def set_expense_item_id(self,expense_item_id):
        """Set expense item id.

        Args:
            expense_item_id(str): Expense item id.

        """
        self.expense_item_id = expense_item_id

    def get_expense_item_id(self):
        """Get expense item id.

        Returns:
            str: Expense item id.

        """
        return self.expense_item_id

    def set_expense_receipt_name(self,expense_receipt_name):
        """Set expense receipt name.

        Args:
            expense_receipt_name(str): Expense receipt name.

        """
        self.expense_receipt_name = expense_receipt_name

    def get_expense_receipt_name(self):
        """Get expense receipt name.

        Returns:
            str: Expense receipt name.

        """
        return self.expense_receipt_name
 
    def set_time_entry_ids(self,time_entry_ids):
        """Set time entry ids.

        Args: 
            time_entry_ids(str): Time entry ids.

        """
        self.time_entry_ids = time_entry_ids

    def get_time_entry_ids(self):
        """Get time entry ids.

        Returns:
            str: Time entry ids.

        """
        return self.time_entry_ids
  
    def set_project_id(self,project_id):
        """Set project id.
 
        Args:
            project_id(str): Project id.

        """
        self.project_id = project_id

    def get_project_id(self):
        """Get project id.
 
        Returns:
            str: Project id.

        """
        return self.project_id
  
    def set_account_id(self,account_id):
        """Set account id.
       
        Args:
            account_id(str): Account id.

        """
        self.account_id = account_id

    def get_account_id(self):
        """Get account id.

        Returns:
            str: Account id.

        """
        return self.account_id
   
    def set_account_name(self,account_name):
        """Set account name.

        Args:
            account_name(str): Account name.
 
        """
        self.account_name = account_name

    def get_account_name(self):
        """Get account name.
 
        Returns:
            str: Account name.

        """
        return self.account_name

    def set_line_id(self,line_id):
        """Set line id.

        Args:
            line_id(str): Line id.

        """
        self.line_id = line_id

    def get_line_id(self):
        """Get line id.

        Returns:
            str: Line id.

        """
        return self.line_id
  
    def set_debit_or_credit(self,debit_or_credit):
        """Set debit or credit.

        Args:
            debit_or_credit(str): Debit or credit.

        """
        self.debit_or_credit = debit_or_credit
 
    def get_debit_or_credit(self):
        """Get debit or credit.

        Returns:
            str: Debit or credit.

        """
        return self.debit_or_credit

    def set_amount(self, amount):
        """Set amount.

        Args: 
            amount(float): Amount.

        """
        self.amount = amount

    def get_amount(self): 
        """Get amount.

        Returns:
            float: Amount.

        """
        return self.amount

    def set_discount_amount(self,discount_amount):
        """Set discount amount.

        Args:
            discount_amount(float): Discount amount.

        """
        self.discount_amount = discount_amount

    def get_discount_amount(self):
        """Get discount amount.

        Returns:
            float: Discount amount.

        """
        return self.discount_amount

    def to_json(self):
        """This method is used to create json object for line items.

        Returns:
            dict: Dictionary containing json object for line items.

        """
        line_item = {}
        if self.item_id != '':
            line_item['item_id'] = self.item_id
        if self.account_id != '':
            line_item['account_id'] = self.account_id
        if self.name != '':
            line_item['name'] = self.name
        if self.description != '':
            line_item['description'] = self.description
        if self.unit != '':
            line_item['unit'] = self.unit
        if self.rate != None:
            line_item['rate'] = self.rate
        if self.item_order != None:
            line_item['item_order'] = self.item_order
        if self.quantity != None:
            line_item['quantity'] = self.quantity
        if self.discount is not None:
            line_item['discount'] = self.discount
        if self.tax_id != '':
            line_item['tax_id'] = self.tax_id
        if self.amount > 0:
            line_item['amount'] = self.amount
        if self.debit_or_credit != '':
            line_item['debit_or_credit'] = self.debit_or_credit
        return line_item
